/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "VDDialogue.h"



/* **********************************************************************************************************************
   Constructors
   ------------
   #param  theID    ID of the dialogue
   #param  theText  Text of the dialogue
   **********************************************************************************************************************/

VDDialogue::VDDialogue() {
    text = NULL;
};

VDDialogue::VDDialogue(const VDDialogue &copyDialogue) {
    text = new VDText();
    text->SetID(copyDialogue.text->GetID());
    text->SetText(copyDialogue.text->GetText());
    text->SetNextID(copyDialogue.text->GetNextID());
    options = copyDialogue.options;
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

VDDialogue::~VDDialogue() {
    FreeMemory();
};



// TODO
void VDDialogue::FreeMemory() {
    if (text != NULL) {
        delete text;
        text = NULL;
    };
};



// TODO

void VDDialogue::AddText(int id, LPCSTR theText) {
    if (text != NULL) {
        delete text;
        text = NULL;
    };
    text = new VDText();
    text->SetID(id);
    text->SetText(theText);
};



/* **********************************************************************************************************************
   AddOption
   ---------
   Adds an option that the user can use to respond to the dialogue.
   #param  theText  Text of the option.
   #param  nextID   ID of the next dialogue that choosing the option results in.
   **********************************************************************************************************************/

void VDDialogue::AddOption(LPCSTR theText, int nextID) {
    VDText newOption;
    newOption.SetText(theText);
    newOption.SetNextID(nextID);
    options.push_back(newOption);
};



/* **********************************************************************************************************************
   RemoveOption
   ------------
   Removes an option from the list.
   #param  text  Text string of the option.
   **********************************************************************************************************************/

void VDDialogue::RemoveOption(LPCSTR text) {
    for (unsigned int n = 0; n < options.size(); n++) {
        if (strcmp(options.at(n).GetText(), text) == 0) {
            std::swap(options.at(n), options.back());
            options.pop_back();
        };
    };
};



/* **********************************************************************************************************************
   WriteToFile
   -----------
   Saves the dialogue to hard disk.
   #param  anOpenFile  Handle to a file that's already open.
   **********************************************************************************************************************/

void VDDialogue::WriteToFile(HANDLE anOpenFile) {
    BOOL returnValue = FALSE;
    size_t numberOfOptions = 0;
    DWORD numberOfBytesWritten = 0;

    // write the text data
    text->WriteToFile(anOpenFile);

    // make sure we let it know how many option elements we're adding
    numberOfOptions = options.size();
    returnValue = WriteFile(anOpenFile, &numberOfOptions, sizeof(numberOfOptions), &numberOfBytesWritten, NULL);
    if (returnValue == FALSE) {
        FreeMemory();
        VDError("Failed to write dialogue data to file.", __FILE__, __LINE__);
    };

    // start writing all the options
    for (unsigned int n = 0; n < numberOfOptions; n++) {
        options.at(n).WriteToFile(anOpenFile);
    };
};



/* **********************************************************************************************************************
   LoadFromFile
   ------------
   Loads the dialogue from hard disk.
   #param  anOpenFile  Handle to a file that's already open.
   **********************************************************************************************************************/

void VDDialogue::LoadFromFile(HANDLE anOpenFile) {
    BOOL returnValue = FALSE;
    DWORD numberOfOptions = 0;
    DWORD numberOfBytesRead = 0;

    // load the text data
    FreeMemory();
    text = new VDText();
    text->LoadFromFile(anOpenFile);

    // load how many option elements we're adding
    returnValue = ReadFile(anOpenFile, &numberOfOptions, sizeof(numberOfOptions), &numberOfBytesRead, NULL);
    if (returnValue == FALSE) {
        FreeMemory();
        VDError("Failed to read dialogue data.", __FILE__, __LINE__);
    };

    // load all the options
    for (unsigned int n = 0; n < numberOfOptions; n++) {
        VDText newOption;
        newOption.LoadFromFile(anOpenFile);
        options.push_back(newOption);
    };
};