/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "VDEntity.h"



/* **********************************************************************************************************************
   Constructor
   **********************************************************************************************************************/

VDEntity::VDEntity() {
    type = 0;
    errno_t returnValue = strcpy_s(name, "");
    if (returnValue != 0) {
        VDError("Failed to set the name of an entity.", __FILE__, __LINE__);
    };
    currentLocationX = 0;
    currentLocationY = 0;
    returnValue = strcpy_s(imageFileLocation, "");
    if (returnValue != 0) {
        VDError("Failed to set the image file location of an entity.", __FILE__, __LINE__);
    };
    direction = 0;
    colorKey = 0xFF000000;
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

VDEntity::~VDEntity() {
};



/* **********************************************************************************************************************
   Move
   ----
   Moves the entity in the specified direction.
   #param  direction  The direction in which to move (1 left, 2 right, 3 up, 4 down)
   **********************************************************************************************************************/

void VDEntity::Move(int direction) {
    switch (direction) {
        case 1:
            currentLocationY -= 1;
            break;
        case 2:
            currentLocationY += 1;
            break;
        case 3:
            currentLocationX -= 1;
            break;
        case 4:
            currentLocationX += 1;
            break;
        default:
            break;
    };
};



/* **********************************************************************************************************************
   WriteToFile
   -----------
   Saves the object to hard disk.
   #param  anOpenFile  Handle to a file that's already open.
   **********************************************************************************************************************/

void VDEntity::WriteToFile(HANDLE anOpenFile) {
    BOOL returnValue = FALSE;
    DWORD numberOfBytesWritten = 0;

    // write the name
    returnValue = WriteFile(anOpenFile, &name, sizeof(name), &numberOfBytesWritten, NULL);
    if (returnValue == FALSE) {
        VDError("Failed to write NPC data to file.", __FILE__, __LINE__);
    };

    // write the x location
    returnValue = WriteFile(anOpenFile, &currentLocationX, sizeof(currentLocationX), &numberOfBytesWritten, NULL);
    if (returnValue == FALSE) {
        VDError("Failed to write NPC data to file.", __FILE__, __LINE__);
    };

    // write the y location
    returnValue = WriteFile(anOpenFile, &currentLocationY, sizeof(currentLocationY), &numberOfBytesWritten, NULL);
    if (returnValue == FALSE) {
        VDError("Failed to write NPC data to file.", __FILE__, __LINE__);
    };

    // write the image file location
    returnValue = WriteFile(anOpenFile, &imageFileLocation, sizeof(imageFileLocation), &numberOfBytesWritten, NULL);
    if (returnValue == FALSE) {
        VDError("Failed to write NPC data to file.", __FILE__, __LINE__);
    };

    // write the direction
    returnValue = WriteFile(anOpenFile, &direction, sizeof(direction), &numberOfBytesWritten, NULL);
    if (returnValue == FALSE) {
        VDError("Failed to write NPC data to file.", __FILE__, __LINE__);
    };

    // write the color key
    returnValue = WriteFile(anOpenFile, &colorKey, sizeof(colorKey), &numberOfBytesWritten, NULL);
    if (returnValue == FALSE) {
        VDError("Failed to write NPC data to file.", __FILE__, __LINE__);
    };
};



/* **********************************************************************************************************************
   LoadFromFile
   ------------
   Loads the object from hard disk.
   #param  anOpenFile  Handle to a file that's already open.
   **********************************************************************************************************************/

void VDEntity::LoadFromFile(HANDLE anOpenFile) {
    BOOL returnValue = FALSE;
    DWORD numberOfBytesRead = 0;

    // load the name
    returnValue = ReadFile(anOpenFile, &name, sizeof(name), &numberOfBytesRead, NULL);
    if (returnValue == FALSE) {
        VDError("Failed to read NPC data.", __FILE__, __LINE__);
    };

    // load the x location
    returnValue = ReadFile(anOpenFile, &currentLocationX, sizeof(currentLocationX), &numberOfBytesRead, NULL);
    if (returnValue == FALSE) {
        VDError("Failed to read NPC data.", __FILE__, __LINE__);
    };

    // load the y location
    returnValue = ReadFile(anOpenFile, &currentLocationY, sizeof(currentLocationY), &numberOfBytesRead, NULL);
    if (returnValue == FALSE) {
        VDError("Failed to read NPC data.", __FILE__, __LINE__);
    };

    // load the image file location
    returnValue = ReadFile(anOpenFile, &imageFileLocation, sizeof(imageFileLocation), &numberOfBytesRead, NULL);
    if (returnValue == FALSE) {
        VDError("Failed to read NPC data.", __FILE__, __LINE__);
    };

    // load the direction
    returnValue = ReadFile(anOpenFile, &direction, sizeof(direction), &numberOfBytesRead, NULL);
    if (returnValue == FALSE) {
        VDError("Failed to read NPC data.", __FILE__, __LINE__);
    };

    // load the color key
    returnValue = ReadFile(anOpenFile, &colorKey, sizeof(colorKey), &numberOfBytesRead, NULL);
    if (returnValue == FALSE) {
        VDError("Failed to read NPC data.", __FILE__, __LINE__);
    };
};