/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "VDError.h"



/* **********************************************************************************************************************
   VDOpenFile
   ----------
   Creates an open file box and sets the file name parameter to the chosen file.
   #param  window    Handle to the window opening the box.
   #param  filter    The type of filter to distinguish which types of files to open.
   #param  fileName  (out) The chosen file location.
   **********************************************************************************************************************/

void VDOpenFile(HWND window, LPSTR filter, LPSTR fileName) {
    char file[FILE_NAME_MAX_SIZE];
    OPENFILENAME openFileBox;

    // zero out the open file box just to be on the safe side
    SecureZeroMemory(&openFileBox, sizeof(OPENFILENAME));

    // set up the open file structure
                openFileBox.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_NOCHANGEDIR;
            openFileBox.hwndOwner = window;
            openFileBox.lpstrFile = file;
         openFileBox.lpstrFile[0] = '\0';
       openFileBox.lpstrFileTitle = NULL;
          openFileBox.lpstrFilter = filter;
      openFileBox.lpstrInitialDir = NULL;
          openFileBox.lStructSize = sizeof(OPENFILENAME);
         openFileBox.nFilterIndex = 1;
             openFileBox.nMaxFile = sizeof(file);
        openFileBox.nMaxFileTitle = NULL;

    // open a file
    BOOL returnValue = GetOpenFileName(&openFileBox);
    if (returnValue != 0) {
        (void) strcpy_s(fileName, sizeof(file), file);
    } else {
        (void) strcpy_s(fileName, sizeof(file), "");
    };
};



/* **********************************************************************************************************************
   VDError
   -------
   Displays an error message.
   #param  message  The error message as described by the engine.
   #param  file     The file in which the error occured.
   #param  line     The line number where the error occured.
   **********************************************************************************************************************/

void VDError(LPCTSTR message, LPCTSTR file, int line) {
    char errorMessage[5000];
    LPTSTR buffer;

    // get the error message from GetLastError()
    (void) FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS, NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &buffer, 0, NULL);

    // set up the full error message
    (void) sprintf_s(errorMessage, "--VDEngine Description--\n%s\n\n--Microsoft Description--\n%s\n\nFILE: %s\nLINE: %d\n", message, buffer, file, line - 2);

    // print the error message
    (void) MessageBox(NULL, errorMessage, "ERROR", MB_TOPMOST | MB_OK | MB_ICONERROR);

    exit(EXIT_FAILURE);
};



/* **********************************************************************************************************************
   VDDirectXError
   --------------
   Displays advanced error information.
   #param  message      The error message as described by the engine.
   #param  returnValue  The return value given by a DirectX function call.
   #param  file         The file in which the error occured.
   #param  line         The line number where the error occured.
   **********************************************************************************************************************/

void VDDirectXError(LPCTSTR message, HRESULT returnValue, LPCTSTR file, int line) {
    char errorMessage[5000];
    LPTSTR buffer;

    // get the error message from GetLastError()
    (void) FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS, NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &buffer, 0, NULL);

    // set up the full error message
    (void) sprintf_s(errorMessage, "--VDEngine Description--\n%s\n\n--DirectX Description--\n\"%s\": %s\n\n--Microsoft Description--\n%s\nFILE: %s\nLINE: %d\n", message, DXGetErrorString(returnValue), DXGetErrorDescription(returnValue), buffer, file, line - 2);

    // print the error message
    (void) MessageBox(NULL, errorMessage, "ERROR", MB_TOPMOST | MB_OK | MB_ICONERROR);

    exit(EXIT_FAILURE);
};