/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "VDItemEditor.h"



/* **********************************************************************************************************************
   Constructor
   **********************************************************************************************************************/

VDItemEditor::VDItemEditor() {
    window = NULL;
    itemListBox = NULL;
    (void) strcpy_s(currentItemSelection, "");
    itemSprite = NULL;
    (void) strcpy_s(itemSpriteImageLocation, "");
    (void) SecureZeroMemory(&itemSpriteDimensions, sizeof(itemSpriteDimensions));
    itemSelected = FALSE;
    removeItemButton = NULL;
    transparencyColor = NULL;
    loadSpriteButton = NULL;
    shiftSpriteUpButton = NULL;
    shiftSpriteDownButton = NULL;
    shiftSpriteLeftButton = NULL;
    shiftSpriteRightButton = NULL;
    nameEditBox = NULL;
    usableOption = NULL;
    armorOption = NULL;
    weaponOption = NULL;
    accessoryOption = NULL;
    questOption = NULL;
    currentItemTypeIndex = 0;

    armorLocationStatic = NULL;
    locationHeadCheckBox = NULL;
    locationEyesCheckBox = NULL;
    locationNeckCheckBox = NULL;
    locationShouldersCheckBox = NULL;
    locationChestCheckBox = NULL;
    locationBackCheckBox = NULL;
    locationAbdomenCheckBox = NULL;
    locationArmsCheckBox = NULL;
    locationHandsCheckBox = NULL;
    locationLegsCheckBox = NULL;
    locationFeetCheckBox = NULL;
    protectionStatic = NULL;
    protectionEditBox = NULL;
    protectionLevelStatic = NULL;
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

VDItemEditor::~VDItemEditor() {
    FreeMemory();
    if (window != NULL) {
        (void) DestroyWindow(window);
        window = NULL;
    };
};



void VDItemEditor::FreeMemory() {
    if (itemSprite != NULL) {
        itemSprite->FreeTexture();
        delete itemSprite;
        itemSprite = NULL;
    };
};



/* **********************************************************************************************************************
   Load
   ----
   Creates a new Item Editor window.
   #param  instance      The instance of the application.
   #param  parentWindow  Handle to the main window (to use for the starting x, y position)
   #param  className     The name of the window class.
   #return  Handle to the window.
   **********************************************************************************************************************/

HWND VDItemEditor::Load(HINSTANCE instance, HWND parentWindow, LPCSTR className) {
    RECT windowDimensions;
    (void) GetWindowRect(parentWindow, &windowDimensions);
    window = CreateWindowEx(WS_EX_TOPMOST | WS_EX_TOOLWINDOW,      // extended window styles
                            className,                             // class name
                            "Item Editor",                         // window name
                            WS_CAPTION | WS_SYSMENU | WS_VISIBLE,  // window styles
                            windowDimensions.left + 100,           // x coordinate (0 = left)
                            windowDimensions.top + 100,            // y coordinate (0 = top)
                            320,                                   // width
                            700,                                   // height
                            NULL,                                  // parent/owner window
                            NULL,                                  // menu
                            instance,                              // instance
                            NULL);                                 // used for CREATESTRUCT structure
    if (window == NULL) {
        FreeMemory();
        VDError("Failed to create the window.", __FILE__, __LINE__);
    };

    // Items
    HWND itemListStatic = CreateWindow("Static", "Items:", WS_CHILD | WS_VISIBLE, 10, 10, 40, 18, window, (HMENU) VD_ITEM_EDITOR_STATIC_ITEM_LIST, instance, NULL);
    if (itemListStatic == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    itemListBox = CreateWindow("ListBox", NULL, WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL, 10, 30, 160, 100, window, (HMENU) VD_ITEM_EDITOR_ITEM_LIST, instance, NULL);
    if (itemListBox == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    (void) strcpy_s(currentItemSelection, "");
    HWND npcNameStatic = CreateWindow("Static", "Name:", WS_CHILD | WS_VISIBLE, 120, 160, 50, 18, window, (HMENU) VD_ITEM_EDITOR_STATIC_NAME, instance, NULL);
    if (npcNameStatic == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    nameEditBox = CreateWindow("Edit", "", WS_CHILD | WS_BORDER | WS_VISIBLE, 120, 180, 180, 18, window, (HMENU) VD_ITEM_EDITOR_NAME, instance, NULL);
    if (nameEditBox == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    loadSpriteButton = CreateWindow("Button", "Load Image", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 120, 205, 80, 20, window, (HMENU) VD_ITEM_EDITOR_LOAD_SPRITE, instance, NULL);
    if (loadSpriteButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND spriteBorderStatic = CreateWindow("Static", "", WS_CHILD | WS_VISIBLE | WS_BORDER, 120, 230, 48, 48, window, (HMENU) VD_ITEM_EDITOR_STATIC_SPRITE_BORDER, instance, NULL);
    if (spriteBorderStatic == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    FreeMemory();
    (void) strcpy_s(itemSpriteImageLocation, "");
    itemSpriteDimensions.left = 128;
    itemSpriteDimensions.top = 238;
    itemSpriteDimensions.right = 160;
    itemSpriteDimensions.bottom = 270;
    shiftSpriteUpButton = CreateWindow("Button", "/\\", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 168, 230, 20, 20, window, (HMENU) VD_ITEM_EDITOR_SHIFT_ITEM_UP, instance, NULL);
    if (shiftSpriteUpButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    shiftSpriteDownButton = CreateWindow("Button", "\\/", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 168, 258, 20, 20, window, (HMENU) VD_ITEM_EDITOR_SHIFT_ITEM_DOWN, instance, NULL);
    if (shiftSpriteDownButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    shiftSpriteLeftButton = CreateWindow("Button", "<", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 120, 278, 20, 20, window, (HMENU) VD_ITEM_EDITOR_SHIFT_ITEM_LEFT, instance, NULL);
    if (shiftSpriteLeftButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    shiftSpriteRightButton = CreateWindow("Button", ">", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 148, 278, 20, 20, window, (HMENU) VD_ITEM_EDITOR_SHIFT_ITEM_RIGHT, instance, NULL);
    if (shiftSpriteRightButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    removeItemButton = CreateWindow("Button", "Remove Item", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 10, 133, 100, 20, window, (HMENU) VD_ITEM_EDITOR_REMOVE_ITEM, instance, NULL);
    if (removeItemButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    // item type
    HWND typeStatic = CreateWindow("Static", "Item Type:", WS_CHILD | WS_VISIBLE, 10, 160, 80, 18, window, (HMENU) VD_ITEM_EDITOR_STATIC_ITEM_TYPE, instance, NULL);
    if (typeStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    usableOption = CreateWindow("Button", "Use", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON | WS_GROUP, 10, 180, 50, 18, window, (HMENU) VD_ITEM_EDITOR_ITEM_OPTION_USE, instance, NULL);
    if (usableOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    armorOption = CreateWindow("Button", "Armor", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 200, 60, 18, window, (HMENU) VD_ITEM_EDITOR_ITEM_OPTION_ARMOR, instance, NULL);
    if (armorOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    weaponOption = CreateWindow("Button", "Weapon", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 220, 75, 18, window, (HMENU) VD_ITEM_EDITOR_ITEM_OPTION_WEAPON, instance, NULL);
    if (weaponOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    accessoryOption = CreateWindow("Button", "Accessory", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 240, 90, 18, window, (HMENU) VD_ITEM_EDITOR_ITEM_OPTION_ACCESSORY, instance, NULL);
    if (accessoryOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    questOption = CreateWindow("Button", "Quest", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 260, 60, 18, window, (HMENU) VD_ITEM_EDITOR_ITEM_OPTION_QUEST, instance, NULL);
    if (questOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    // save items
    saveChangesButton = CreateWindow("Button", "SAVE CHANGES", WS_CHILD | BS_PUSHBUTTON | WS_BORDER | WS_VISIBLE, 60, 560, 200, 40, window, (HMENU) VD_ITEM_EDITOR_SAVE_CHANGES, instance, NULL);
    if (saveChangesButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    return window;
};



/* **********************************************************************************************************************
   ProcessMessages
   ---------------
   Processes messages sent by and received from the Item Editor.
   #param TODO
   **********************************************************************************************************************/

void VDItemEditor::ProcessMessages(HINSTANCE instance, LPDIRECT3DDEVICE9 direct3DDevice, VDInput* input, vector<VDItem>* items) {
    // set the input to the Item Editor
    if (input->GetCurrentWindow() != window) {
        input->ChangeInputWindow(window);
    };

    // populate the item list if it's empty
    if (SendMessage(itemListBox, LB_GETCOUNT, 0, 0) == 0) {
        for (size_t n = 0; n < items->size(); n++) {
            (void) SendMessage(itemListBox, LB_ADDSTRING, 0, (LPARAM) items->at(n).GetName());
        };
    };

    // Load Sprite button
    /////////////////////

    if (SendMessage(loadSpriteButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
        char text[FILE_NAME_MAX_SIZE];

        // open a sprite image
        VDOpenFile(window, "Sprites (*.png,*.bmp)\0*.png;*.bmp\0All Files\0*.*\0", text);

        // make sure the user selected a file
        if (strcmp(text, "") != 0) {
            // set the sprite file location
            (void) strcpy_s(itemSpriteImageLocation, text);

            // add the sprite to the editor so that it may be drawn
            RECT drawingArea;
            (void) SetRect(&drawingArea, 0, 0, 32, 32);
            FreeMemory();
            itemSprite = new VDSprite(direct3DDevice, text, drawingArea, D3DXVECTOR3((float) itemSpriteDimensions.left, (float) itemSpriteDimensions.top, 0.0f), 0xFF000000);
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Up button
    ////////////

    if (SendMessage(shiftSpriteUpButton, BM_GETSTATE, 0, 0) == BST_FOCUS  || input->GetKeyboardInput()[DIK_UP] & 0x80) {
        if (itemSprite != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) itemSprite->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            if (drawingArea.top == 0) {
                LONG bottom = itemSprite->GetSpriteHeight();
                itemSprite->SetSourceRectangle(drawingArea.left, bottom - 32, drawingArea.right, bottom);
            } else {
                itemSprite->SetSourceRectangle(drawingArea.left, drawingArea.top - 32, drawingArea.right, drawingArea.bottom - 32);
            };
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Down button
    //////////////

    if (SendMessage(shiftSpriteDownButton, BM_GETSTATE, 0, 0) == BST_FOCUS || input->GetKeyboardInput()[DIK_DOWN] & 0x80) {
        if (itemSprite != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) itemSprite->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            LONG bottom = itemSprite->GetSpriteHeight();
            if (drawingArea.bottom == bottom) {
                itemSprite->SetSourceRectangle(drawingArea.left, 0, drawingArea.right, 32);
            } else {
                itemSprite->SetSourceRectangle(drawingArea.left, drawingArea.top + 32, drawingArea.right, drawingArea.bottom + 32);
            };
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Left button
    //////////////

    if (SendMessage(shiftSpriteLeftButton, BM_GETSTATE, 0, 0) == BST_FOCUS || input->GetKeyboardInput()[DIK_LEFT] & 0x80) {
        if (itemSprite != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) itemSprite->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            if (drawingArea.left == 0) {
                LONG right = itemSprite->GetSpriteWidth();
                itemSprite->SetSourceRectangle(right - 32, drawingArea.top, right, drawingArea.bottom);
            } else {
                itemSprite->SetSourceRectangle(drawingArea.left - 32, drawingArea.top, drawingArea.right - 32, drawingArea.bottom);
            };
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Right button
    ///////////////

    if (SendMessage(shiftSpriteRightButton, BM_GETSTATE, 0, 0) == BST_FOCUS || input->GetKeyboardInput()[DIK_RIGHT] & 0x80) {
        if (itemSprite != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) itemSprite->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            LONG right = itemSprite->GetSpriteWidth();
            if (drawingArea.right == right) {
                itemSprite->SetSourceRectangle(0, drawingArea.top, 32, drawingArea.bottom);
            } else {
                itemSprite->SetSourceRectangle(drawingArea.left + 32, drawingArea.top, drawingArea.right + 32, drawingArea.bottom);
            };
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Add Item button
    //////////////////

    if (SendMessage(addItemButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
        // armor
        if (SendMessage(armorOption, BM_GETCHECK, 0, 0) == BST_CHECKED) {
            char name[FILE_NAME_MAX_SIZE];

            // get the name, and image
            (void) GetWindowText(nameEditBox, name, sizeof(name));

            // get the coverage locations
            int head = 0, eyes = 0, neck = 0, shoulders = 0, chest = 0, back = 0, abdomen = 0, arms = 0, hands = 0, legs = 0, feet = 0;
            BOOL covered = FALSE;
            if (SendMessage(locationHeadCheckBox, BM_GETCHECK, 0, 0) == BST_CHECKED) {
                head = 1;
                covered = TRUE;
            };
            if (SendMessage(locationEyesCheckBox, BM_GETCHECK, 0, 0) == BST_CHECKED) {
                eyes = 1;
                covered = TRUE;
            };
            if (SendMessage(locationNeckCheckBox, BM_GETCHECK, 0, 0) == BST_CHECKED) {
                neck = 1;
                covered = TRUE;
            };
            if (SendMessage(locationShouldersCheckBox, BM_GETCHECK, 0, 0) == BST_CHECKED) {
                shoulders = 1;
                covered = TRUE;
            };
            if (SendMessage(locationChestCheckBox, BM_GETCHECK, 0, 0) == BST_CHECKED) {
                chest = 1;
                covered = TRUE;
            };
            if (SendMessage(locationBackCheckBox, BM_GETCHECK, 0, 0) == BST_CHECKED) {
                back = 1;
                covered = TRUE;
            };
            if (SendMessage(locationAbdomenCheckBox, BM_GETCHECK, 0, 0) == BST_CHECKED) {
                abdomen = 1;
                covered = TRUE;
            };
            if (SendMessage(locationArmsCheckBox, BM_GETCHECK, 0, 0) == BST_CHECKED) {
                arms = 1;
                covered = TRUE;
            };
            if (SendMessage(locationHandsCheckBox, BM_GETCHECK, 0, 0) == BST_CHECKED) {
                hands = 1;
                covered = TRUE;
            };
            if (SendMessage(locationLegsCheckBox, BM_GETCHECK, 0, 0) == BST_CHECKED) {
                legs = 1;
                covered = TRUE;
            };
            if (SendMessage(locationFeetCheckBox, BM_GETCHECK, 0, 0) == BST_CHECKED) {
                feet = 1;
                covered = TRUE;
            };

            // check for name, any number of coverage locations, and image
            if (strcmp(name, "") != 0 && itemSprite != NULL && covered == TRUE) {
                char newImageFile[FILE_NAME_MAX_SIZE], extension[10];

                // remove the path name, save the extension
                int n = 0, m = 0;
                while (itemSpriteImageLocation[n] != '.') {
                    n++;
                };
                while (itemSpriteImageLocation[n] != '\0') {
                    extension[m] = itemSpriteImageLocation[n];
                    n++;
                    m++;
                };
                extension[m] = '\0';

                // use the item name for the new texture
                (void) strcpy_s(newImageFile, "Images/");
                (void) strcat_s(newImageFile, name);
                (void) strcat_s(newImageFile, extension);

                // create the texture
                CreateItemTexture(direct3DDevice, itemSprite, newImageFile);

                // get protection level
                char text[FILE_NAME_MAX_SIZE];
                (void) GetWindowText(protectionEditBox, text, sizeof(text));
                int protectionLevel = atoi(text);

                // add the item
                VDItem newItem = VDItem();
                newItem.SetName(name);
                newItem.SetType(2); // 0 null, 1 usable, 2 armor, 3 weapon, 4 accessory, 5 quest
                newItem.SetImageFileLocation(itemSpriteImageLocation);
                newItem.SetCoverage(head, eyes, neck, shoulders, chest, back, abdomen, arms, hands, legs, feet);
                newItem.SetProtection(protectionLevel);
                items->push_back(newItem);

                // update the window
                (void) SendMessage(itemListBox, LB_ADDSTRING, 0, (LPARAM) name);
            };
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Usable radio button
    //////////////////////

    if (SendMessage(usableOption, BM_GETCHECK, 0, 0) == BST_CHECKED) {
        // user selects a different item type
        if (currentItemTypeIndex != 1) {
            currentItemTypeIndex = 1;

            // clear the data
            ResetWindow();

            // populate the window
        };
    };

    // Armor radio button
    /////////////////////

    if (SendMessage(armorOption, BM_GETCHECK, 0, 0) == BST_CHECKED) {
        if (currentItemTypeIndex != 2) {
            currentItemTypeIndex = 2;

            // clear the daat
            ResetWindow();

            // populate the window
            armorLocationStatic = CreateWindow("Static", "Coverage:", WS_CHILD | WS_VISIBLE, 10, 290, 110, 18, window, (HMENU) VD_ITEM_EDITOR_STATIC_LOCATION, instance, NULL);
            if (armorLocationStatic == NULL) {
                FreeMemory();
                VDError("Failed to create an interface resource.", __FILE__, __LINE__);
            };
            locationHeadCheckBox = CreateWindow("Button", "Head", WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX, 10, 310, 55, 18, window, (HMENU) VD_ITEM_EDITOR_LOCATION_HEAD, instance, NULL);
            locationEyesCheckBox = CreateWindow("Button", "Eyes", WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX, 10, 330, 55, 18, window, (HMENU) VD_ITEM_EDITOR_LOCATION_EYES, instance, NULL);
            locationNeckCheckBox = CreateWindow("Button", "Neck", WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX, 10, 350, 55, 18, window, (HMENU) VD_ITEM_EDITOR_LOCATION_NECK, instance, NULL);
            locationShouldersCheckBox = CreateWindow("Button", "Shoulders", WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX, 10, 370, 90, 18, window, (HMENU) VD_ITEM_EDITOR_LOCATION_SHOULDERS, instance, NULL);
            locationChestCheckBox = CreateWindow("Button", "Chest", WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX, 10, 390, 60, 18, window, (HMENU) VD_ITEM_EDITOR_LOCATION_CHEST, instance, NULL);
            locationBackCheckBox = CreateWindow("Button", "Back", WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX, 10, 410, 55, 18, window, (HMENU) VD_ITEM_EDITOR_LOCATION_BACK, instance, NULL);
            locationAbdomenCheckBox = CreateWindow("Button", "Abdomen", WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX, 100, 310, 80, 18, window, (HMENU) VD_ITEM_EDITOR_LOCATION_ABDOMEN, instance, NULL);
            locationArmsCheckBox = CreateWindow("Button", "Arms", WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX, 100, 330, 55, 18, window, (HMENU) VD_ITEM_EDITOR_LOCATION_ARMS, instance, NULL);
            locationHandsCheckBox = CreateWindow("Button", "Hands", WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX, 100, 350, 60, 18, window, (HMENU) VD_ITEM_EDITOR_LOCATION_HANDS, instance, NULL);
            locationLegsCheckBox = CreateWindow("Button", "Legs", WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX, 100, 370, 50, 18, window, (HMENU) VD_ITEM_EDITOR_LOCATION_LEGS, instance, NULL);
            locationFeetCheckBox = CreateWindow("Button", "Feet", WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX, 100, 390, 50, 18, window, (HMENU) VD_ITEM_EDITOR_LOCATION_FEET, instance, NULL);
            protectionStatic = CreateWindow("Static", "Protection:", WS_CHILD | WS_VISIBLE, 10, 430, 80, 18, window, (HMENU) VD_ITEM_EDITOR_STATIC_PROTECTION, instance, NULL);
            protectionEditBox = CreateWindow("Edit", "0", WS_CHILD | WS_VISIBLE | WS_BORDER, 100, 430, 50, 18, window, (HMENU) VD_ITEM_EDITOR_PROTECTION_EDIT_BOX, instance, NULL);
            protectionLevelStatic = CreateWindow("Static", "none", WS_CHILD | WS_VISIBLE | WS_BORDER, 160, 430, 100, 18, window, (HMENU) VD_ITEM_EDITOR_STATIC_PROTECTION_LEVEL, instance, NULL);
            addItemButton = CreateWindow("Button", "Add Item", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 10, 450, 80, 20, window, (HMENU) VD_ITEM_EDITOR_SAVE_ITEM, instance, NULL);
            if (addItemButton == NULL) {
                FreeMemory();
                VDError("Failed to create an interface resource.", __FILE__, __LINE__);
            };
        };

        // update the protection level description
        char text[FILE_NAME_MAX_SIZE];
        (void) GetWindowText(protectionEditBox, text, sizeof(text));
        int protectionLevel = atoi(text);
        if (protectionLevel == 0) {
            (void) SetWindowText(protectionLevelStatic, "none");
        } else if (protectionLevel >= 1 && protectionLevel <= 10) {
            (void) SetWindowText(protectionLevelStatic, "poor");
        } else if (protectionLevel >= 11 && protectionLevel <= 20) {
            (void) SetWindowText(protectionLevelStatic, "very little");
        } else if (protectionLevel >= 21 && protectionLevel <= 30) {
            (void) SetWindowText(protectionLevelStatic, "a little");
        } else if (protectionLevel >= 31 && protectionLevel <= 40) {
            (void) SetWindowText(protectionLevelStatic, "some");
        } else if (protectionLevel >= 41 && protectionLevel <= 50) {
            (void) SetWindowText(protectionLevelStatic, "low");
        } else if (protectionLevel >= 51 && protectionLevel <= 60) {
            (void) SetWindowText(protectionLevelStatic, "fair");
        } else if (protectionLevel >= 61 && protectionLevel <= 70) {
            (void) SetWindowText(protectionLevelStatic, "decent");
        } else if (protectionLevel >= 71 && protectionLevel <= 80) {
            (void) SetWindowText(protectionLevelStatic, "moderate");
        } else if (protectionLevel >= 81 && protectionLevel <= 90) {
            (void) SetWindowText(protectionLevelStatic, "good");
        } else if (protectionLevel >= 91 && protectionLevel <= 100) {
            (void) SetWindowText(protectionLevelStatic, "great");
        } else if (protectionLevel >= 101 && protectionLevel <= 110) {
            (void) SetWindowText(protectionLevelStatic, "high");
        } else if (protectionLevel >= 111 && protectionLevel <= 120) {
            (void) SetWindowText(protectionLevelStatic, "incredible");
        } else if (protectionLevel >= 121) {
            (void) SetWindowText(protectionLevelStatic, "god-like");
        };
    };

    // Weapon radio button
    //////////////////////

    if (SendMessage(weaponOption, BM_GETCHECK, 0, 0) == BST_CHECKED) {
        // user selects a different item type
        if (currentItemTypeIndex != 3) {
            currentItemTypeIndex = 3;

            // clear the data
            ResetWindow();

            // populate the window
        };
    };

    // Accessory radio button
    /////////////////////////

    if (SendMessage(accessoryOption, BM_GETCHECK, 0, 0) == BST_CHECKED) {
        // user selects a different item type
        if (currentItemTypeIndex != 4) {
            currentItemTypeIndex = 4;

            // clear the data
            ResetWindow();

            // populate the window
        };
    };

    // Quest radio button
    /////////////////////

    if (SendMessage(questOption, BM_GETCHECK, 0, 0) == BST_CHECKED) {
        // user selects a different item type
        if (currentItemTypeIndex != 5) {
            currentItemTypeIndex = 5;

            // clear the data
            ResetWindow();

            // populate the window
        };
    };

    // Save Changes button
    //////////////////////

    if (SendMessage(saveChangesButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
        // save the NPCs
        WriteItems(items);

        // lets other editors know we've changed NPC's
        reloadItems = TRUE;

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };
};



// TODO
void VDItemEditor::ResetWindow() {
    if (armorLocationStatic != NULL) {
        (void) DestroyWindow(armorLocationStatic);
    };
    if (locationHeadCheckBox != NULL) {
        (void) DestroyWindow(locationHeadCheckBox);
    };
    if (locationEyesCheckBox != NULL) {
        (void) DestroyWindow(locationEyesCheckBox);
    };
    if (locationNeckCheckBox != NULL) {
        (void) DestroyWindow(locationNeckCheckBox);
    };
    if (locationShouldersCheckBox != NULL) {
        (void) DestroyWindow(locationShouldersCheckBox);
    };
    if (locationChestCheckBox != NULL) {
        (void) DestroyWindow(locationChestCheckBox);
    };
    if (locationBackCheckBox != NULL) {
        (void) DestroyWindow(locationBackCheckBox);
    };
    if (locationAbdomenCheckBox != NULL) {
        (void) DestroyWindow(locationAbdomenCheckBox);
    };
    if (locationArmsCheckBox != NULL) {
        (void) DestroyWindow(locationArmsCheckBox);
    };
    if (locationHandsCheckBox != NULL) {
        (void) DestroyWindow(locationHandsCheckBox);
    };
    if (locationLegsCheckBox != NULL) {
        (void) DestroyWindow(locationLegsCheckBox);
    };
    if (locationFeetCheckBox != NULL) {
        (void) DestroyWindow(locationFeetCheckBox);
    };
    if (protectionStatic != NULL) {
        (void) DestroyWindow(protectionStatic);
    };
    if (protectionEditBox != NULL) {
        (void) DestroyWindow(protectionEditBox);
    };
    if (protectionLevelStatic != NULL) {
        (void) DestroyWindow(protectionLevelStatic);
    };
    if (addItemButton != NULL) {
        (void) DestroyWindow(addItemButton);
    };
};



// TODO
void VDItemEditor::CreateItemTexture(LPDIRECT3DDEVICE9 direct3DDevice, VDSprite* sprite, LPCSTR newFileName) {
    RECT sourceRect;
    LPDIRECT3DTEXTURE9 tempTexture = NULL;

    // get the extension
    char extension[10];
    int n = 0, m = 0;
    while (newFileName[n] != '.') {
        n++;
    };
    while (newFileName[n] != '\0') {
        extension[m] = newFileName[n];
        n++;
        m++;
    };
    extension[m] = '\0';

    // get the width of the sprite
    UINT sourceSpriteWidth = sprite->GetSpriteWidth();

    // create a texture that's the width of the sprite and 32 pixels in height
    HRESULT returnValue = D3DXCreateTexture(direct3DDevice, 32, 32, D3DX_DEFAULT, 0, D3DFMT_X8R8G8B8, D3DPOOL_SCRATCH, &tempTexture);

    // lock the textures
    D3DLOCKED_RECT sourceSprite, destSprite;
    returnValue = sprite->GetTexture()->LockRect(0, &sourceSprite, NULL, D3DLOCK_READONLY);
    if (FAILED(returnValue)) {
        FreeMemory();
        VDDirectXError("Failed to lock the texture.", returnValue, __FILE__, __LINE__);
    }
    returnValue = tempTexture->LockRect(0, &destSprite, NULL, D3DLOCK_DISCARD);
    if (FAILED(returnValue)) {
        sprite->GetTexture()->UnlockRect(0);
        FreeMemory();
        VDDirectXError("Failed to lock the texture.", returnValue, __FILE__, __LINE__);
    };

    // copy the data
    UINT bytesPerPixel = 4;
    sprite->GetSourceRectangle(&sourceRect);
    const BYTE* sourceLoc = (const BYTE*) sourceSprite.pBits + sourceRect.top * sourceSpriteWidth * bytesPerPixel
                                                             + sourceRect.left * bytesPerPixel;
    for (int n = 0; n < 32; n++) {
        BYTE* destAddress = (BYTE *) destSprite.pBits + n * destSprite.Pitch;
        memcpy(destAddress, sourceLoc, 32 * bytesPerPixel);
        sourceLoc += sourceSprite.Pitch;
    };

    // unlock the textures
    sprite->GetTexture()->UnlockRect(0);
    tempTexture->UnlockRect(0);

    // save the texture to hard disk - requires a directory Images to be in the app directory
    if (strcmp(extension, ".png") == 0) {
        returnValue = D3DXSaveTextureToFile(newFileName, D3DXIFF_PNG, tempTexture, NULL);
        if (FAILED(returnValue)) {
            FreeMemory();
            VDDirectXError("Failed to save texture.", returnValue, __FILE__, __LINE__);
        };
    } else if (strcmp(extension, ".bmp") == 0) {
        returnValue = D3DXSaveTextureToFile(newFileName, D3DXIFF_BMP, tempTexture, NULL);
        if (FAILED(returnValue)) {
            FreeMemory();
            VDDirectXError("Failed to save texture.", returnValue, __FILE__, __LINE__);
        };
    };

    // free memory
    if (tempTexture != NULL) {
        tempTexture->Release();
        tempTexture = NULL;
    };
};



/* **********************************************************************************************************************
   WriteItems
   ----------
   #param  items  Pointer to the list of items.
   Saves the items to hard disk.
   **********************************************************************************************************************/

void VDItemEditor::WriteItems(vector<VDItem>* items) {
    HANDLE file = NULL;
    BOOL returnValue = FALSE;
    DWORD numberOfBytesWritten = 0;
    size_t numberOfItems = 0;

    // open the file
    file = CreateFile("Data/Items.vd", GENERIC_WRITE, FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    if (file == INVALID_HANDLE_VALUE) {
        FreeMemory();
        VDError("Failed to create the Items.vd file.", __FILE__, __LINE__);
    };

    // make sure we let it know how many item elements we're adding
    numberOfItems = items->size();
    returnValue = WriteFile(file, &numberOfItems, sizeof(numberOfItems), &numberOfBytesWritten, NULL);
    if (returnValue == FALSE) {
        FreeMemory();
        VDError("Failed to write item data to file.", __FILE__, __LINE__);
    };

    // write all the items
    for (size_t n = 0; n < numberOfItems; n++) {
        items->at(n).WriteToFile(file);
        if (returnValue == FALSE) {
            FreeMemory();
            VDError("Failed to write item data to file.", __FILE__, __LINE__);
        };
    };

    // close the file
    returnValue = CloseHandle(file);
    if (returnValue == FALSE) {
        FreeMemory();
        VDError("Failed to close the Items.vd file.", __FILE__, __LINE__);
    };
};



VDItem* VDItemEditor::GetVDItem(vector<VDItem>* items, LPCSTR name) {
    for (unsigned int n = 0; n < items->size(); n++) {
        if (strcmp(name, items->at(n).GetName()) == 0) {
            return &items->at(n);
        };
    };

    return NULL;
};