/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "VDMapEditor.h"



/* **********************************************************************************************************************
   Constructor
   **********************************************************************************************************************/

VDMapEditor::VDMapEditor() {
    window = NULL;
    transparencyColor = NULL;
    loadSpriteButton = NULL;
    tileSheet = NULL;
    (void) strcpy_s(tileSheetImageLocation, "");
    (void) SecureZeroMemory(&tileSheetDimensions, sizeof(tileSheetDimensions));
    (void) strcpy_s(selectedTileImageLocation, "");
    (void) SecureZeroMemory(&tileSheetSourceRectangle, sizeof(tileSheetSourceRectangle));
    shiftTileSpriteUp = NULL;
    shiftTileSpriteDown = NULL;
    shiftTileSpriteLeft = NULL;
    shiftTileSpriteRight = NULL;
    tileSpawnsListBox = NULL;
    creaturesListBox = NULL;
    removeSpawnButton = NULL;
    addSpawnButton = NULL;
    nullTileOption = NULL;
    passableOverTileOption = NULL;
    passableUnderTileOption = NULL;
    impassableTileOption = NULL;
    newAreaTileOption = NULL;
    loadNewAreaButton = NULL;
    newAreaStatic = NULL;
    (void) strcpy_s(newAreaFileLocation, "");
    newAreaStartingPointX = NULL;
    newAreaStartingPointY = NULL;
    objectListBox = NULL;
    objectSprite = NULL;
    (void) strcpy_s(selectedObjectImageLocation, "");
    (void) strcpy_s(previousObjectSelection, "");
    (void) SecureZeroMemory(&objectSpriteDimensions, sizeof(objectSpriteDimensions));
    previousAnimationButton = NULL;
    nextAnimationButton = NULL;
    nullObjectOption = NULL;
    npcOption = NULL;
    passableOverObjectOption = NULL;
    passableUnderObjectOption = NULL;
    impassableObjectOption = NULL;
    tileSelected = FALSE;
    selectedRadioButton = 0;
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

VDMapEditor::~VDMapEditor() {
    FreeMemory();
};



// TODO - fill Free Memory on all errors
void VDMapEditor::FreeMemory() {
    if (objectSprite != NULL) {
        objectSprite->FreeTexture();
        delete objectSprite;
        objectSprite = NULL;
    };
    if (tileSheet != NULL) {
        tileSheet->FreeTexture();
        delete tileSheet;
        tileSheet = NULL;
    };
};



/* **********************************************************************************************************************
   Load
   ----
   Creates a new Map Editor window.
   #param  instance      The instance of the application.
   #param  parentWindow  Handle to the main window (to use for the starting x, y position)
   #param  className     The name of the window class.
   #return  Handle to the window.
   **********************************************************************************************************************/

HWND VDMapEditor::Load(HINSTANCE instance, HWND parentWindow, LPCSTR className) {
    // create the menu resource
    HMENU menu = LoadMenu(instance, MAKEINTRESOURCE(VD_MENU));
    if (menu == NULL) {
        VDError("Failed to create the menu resource.", __FILE__, __LINE__);
    };

    // create the window
    RECT windowDimensions;
    (void) GetWindowRect(parentWindow, &windowDimensions);
    window = CreateWindowEx(WS_EX_TOPMOST | WS_EX_TOOLWINDOW,  // extended window styles
                            className,                         // class name
                            "Map Editor",                      // window name
                            WS_CAPTION | WS_VISIBLE,           // window styles
                            windowDimensions.left + 100,       // x coordinate (0 = left)
                            windowDimensions.top + 100,        // y coordinate (0 = top)
                            277,                               // width
                            915,                               // height
                            NULL,                              // parent/owner window
                            menu,                              // menu
                            instance,                          // instance
                            NULL);                             // used for CREATESTRUCT structure
    if (window == NULL) {
        VDError("Failed to create the map editor window.", __FILE__, __LINE__);
    };

    FreeMemory();

    HWND transparentColorStatic = CreateWindow("Static", "Transparency Color:", WS_CHILD | WS_VISIBLE, 10, 10, 135, 18, window, (HMENU) VD_MAP_EDITOR_STATIC_TRANSPARENCY, instance, NULL);
    if (transparentColorStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    transparencyColor = CreateWindow("Edit", "0xFF000000", WS_CHILD | WS_VISIBLE | WS_BORDER, 150, 10, 95, 18, window, (HMENU) VD_MAP_EDITOR_TRANSPARENCY, instance, NULL);
    if (transparencyColor == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND divider2 = CreateWindow("Static", "----------------------------------------------------------------------", WS_CHILD | WS_VISIBLE, 10, 30, 250, 10, window, (HMENU) VD_MAP_EDITOR_DIVIDER2, instance, NULL);
    if (divider2 == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND tilesStatic = CreateWindow("Static", " TILES", WS_CHILD | WS_VISIBLE | WS_BORDER, 110, 45, 50, 18, window, (HMENU) VD_MAP_EDITOR_STATIC_TILES, instance, NULL);
    if (tilesStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    loadSpriteButton = CreateWindow("Button", "Load Sprite Sheet", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 10, 70, 125, 20, window, (HMENU) VD_MAP_EDITOR_LOAD_SPRITE, instance, NULL);
    if (loadSpriteButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    (void) strcpy_s(tileSheetImageLocation, "");
    (void) SetRect(&tileSheetDimensions, 10, 95, 170, 255);
    (void) strcpy_s(selectedTileImageLocation, "");
    (void) SetRect(&tileSheetSourceRectangle, 0, 0, 32, 32);
    shiftTileSpriteUp = CreateWindow("Button", "/\\", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 170, 95, 20, 20, window, (HMENU) VD_MAP_EDITOR_UP, instance, NULL);
    if (shiftTileSpriteUp == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    shiftTileSpriteDown = CreateWindow("Button", "\\/", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 170, 235, 20, 20, window, (HMENU) VD_MAP_EDITOR_DOWN, instance, NULL);
    if (shiftTileSpriteDown == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    shiftTileSpriteLeft = CreateWindow("Button", "<", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 10, 255, 20, 20, window, (HMENU) VD_MAP_EDITOR_LEFT, instance, NULL);
    if (shiftTileSpriteLeft == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    shiftTileSpriteRight = CreateWindow("Button", ">", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 150, 255, 20, 20, window, (HMENU) VD_MAP_EDITOR_RIGHT, instance, NULL);
    if (shiftTileSpriteRight == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND tileSpawnsStatic = CreateWindow("Static", "Tile Spawns:", WS_CHILD | WS_VISIBLE, 10, 280, 100, 18, window, (HMENU) VD_MAP_EDITOR_STATIC_TILE_SPAWNS, instance, NULL);
    if (tileSpawnsStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    tileSpawnsListBox = CreateWindow("ListBox", "", WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL, 10, 300, 120, 100, window, (HMENU) VD_MAP_EDITOR_TILE_SPAWNS, instance, NULL);
    if (tileSpawnsListBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND creaturesStatic = CreateWindow("Static", "Creature:", WS_CHILD | WS_VISIBLE, 140, 280, 80, 18, window, (HMENU) VD_MAP_EDITOR_STATIC_CREATURES, instance, NULL);
    if (creaturesStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    creaturesListBox = CreateWindow("ListBox", "", WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL, 140, 300, 120, 100, window, (HMENU) VD_MAP_EDITOR_CREATURES, instance, NULL);
    if (creaturesListBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    removeSpawnButton = CreateWindow("Button", "Remove Spawn", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 10, 405, 110, 20, window, (HMENU) VD_MAP_EDITOR_REMOVE_SPAWN, instance, NULL);
    if (removeSpawnButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    addSpawnButton = CreateWindow("Button", "Add Spawn", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 140, 405, 80, 20, window, (HMENU) VD_MAP_EDITOR_ADD_SPAWN, instance, NULL);
    if (addSpawnButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND tileTypesStatic = CreateWindow("Static", "Tile Types:", WS_CHILD | WS_VISIBLE, 10, 430, 80, 18, window, (HMENU) VD_MAP_EDITOR_STATIC_TILE_TYPES, instance, NULL);
    if (tileTypesStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    nullTileOption = CreateWindow("Button", "Remove", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON | WS_GROUP, 10, 450, 80, 18, window, (HMENU) VD_MAP_EDITOR_TILE_OPTION_NULL, instance, NULL);
    if (nullTileOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    (void) SendMessage(nullTileOption, BM_SETCHECK, BST_CHECKED, 0);
    selectedRadioButton = 0;
    passableOverTileOption = CreateWindow("Button", "Passable (entities appear on top)", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 470, 240, 18, window, (HMENU) VD_MAP_EDITOR_TILE_OPTION_PASSABLE_OVER, instance, NULL);
    if (passableOverTileOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    passableUnderTileOption = CreateWindow("Button", "Passable (entities are hidden)", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 490, 220, 18, window, (HMENU) VD_MAP_EDITOR_TILE_OPTION_PASSABLE_UNDER, instance, NULL);
    if (passableOverTileOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    impassableTileOption = CreateWindow("Button", "Impassable", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 510, 100, 18, window, (HMENU) VD_MAP_EDITOR_TILE_OPTION_IMPASSABLE, instance, NULL);
    if (impassableTileOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    newAreaTileOption = CreateWindow("Button", "New Area", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 530, 80, 18, window, (HMENU) VD_MAP_EDITOR_TILE_OPTION_NEW_AREA, instance, NULL);
    if (newAreaTileOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    loadNewAreaButton = CreateWindow("Button", "Load New Area", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 10, 555, 110, 20, window, (HMENU) VD_MAP_EDITOR_LOAD_NEW_AREA, instance, NULL);
    if (loadNewAreaButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    newAreaStatic = CreateWindow("Static", "", WS_CHILD | WS_VISIBLE | WS_BORDER, 10, 580, 120, 18, window, (HMENU) VD_MAP_EDITOR_STATIC_NEW_AREA, instance, NULL);
    if (newAreaStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    (void) strcpy_s(newAreaFileLocation, "");
    HWND newAreaStartingPointStatic = CreateWindow("Static", "Starting Point:", WS_CHILD | WS_VISIBLE, 140, 560, 120, 18, window, (HMENU) VD_MAP_EDITOR_STATIC_STARTING_POINT, instance, NULL);
    if (newAreaStartingPointStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    newAreaStartingPointX = CreateWindow("Edit", "0", WS_CHILD | WS_VISIBLE | WS_BORDER, 140, 580, 30, 18, window, (HMENU) VD_MAP_EDITOR_STARTING_POINT_X, instance, NULL);
    if (newAreaStartingPointX == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    newAreaStartingPointY = CreateWindow("Edit", "0", WS_CHILD | WS_VISIBLE | WS_BORDER, 180, 580, 30, 18, window, (HMENU) VD_MAP_EDITOR_STARTING_POINT_Y, instance, NULL);
    if (newAreaStartingPointY == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND divider = CreateWindow("Static", "----------------------------------------------------------------------", WS_CHILD | WS_VISIBLE, 10, 600, 250, 10, window, (HMENU) VD_MAP_EDITOR_DIVIDER, instance, NULL);
    if (divider == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    // add object
    HWND addNPCStatic = CreateWindow("Static", " OBJECTS", WS_CHILD | WS_VISIBLE | WS_BORDER, 100, 615, 73, 18, window, (HMENU) VD_MAP_EDITOR_STATIC_ADD_NPC, instance, NULL);
    if (addNPCStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    objectListBox = CreateWindow("ListBox", "", WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL, 10, 640, 120, 100, window, (HMENU) VD_MAP_EDITOR_NPCS, instance, NULL);
    if (objectListBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND spriteBorderStatic = CreateWindow("Static", "", WS_CHILD | WS_VISIBLE | WS_BORDER, 140, 640, 48, 48, window, (HMENU) VD_MAP_EDITOR_STATIC_SPRITE_BORDER, instance, NULL);
    if (spriteBorderStatic == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    (void) strcpy_s(selectedObjectImageLocation, "");
    (void) strcpy_s(previousObjectSelection, "");
    (void) SetRect(&objectSpriteDimensions, 148, 648, 180, 680);
    previousAnimationButton = CreateWindow("Button", "<", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 140, 688, 20, 20, window, (HMENU) VD_MAP_EDITOR_PREVIOUS_ANIMATION, instance, NULL); 
    if (previousAnimationButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    nextAnimationButton = CreateWindow("Button", ">", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 168, 688, 20, 20, window, (HMENU) VD_MAP_EDITOR_PREVIOUS_ANIMATION, instance, NULL); 
    if (nextAnimationButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND objectTypesStatic = CreateWindow("Static", "Object Types:", WS_CHILD | WS_VISIBLE, 10, 745, 90, 18, window, (HMENU) VD_MAP_EDITOR_STATIC_OBJECT_TYPES, instance, NULL);
    if (objectTypesStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    nullObjectOption = CreateWindow("Button", "Remove", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 765, 80, 18, window, (HMENU) VD_MAP_EDITOR_OBJECT_OPTION_NULL, instance, NULL);
    if (nullObjectOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    npcOption = CreateWindow("Button", "NPC", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 785, 50, 18, window, (HMENU) VD_MAP_EDITOR_OBJECT_OPTION_NPC, instance, NULL);
    if (npcOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    passableOverObjectOption = CreateWindow("Button", "Passable (entities appear on top)", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 805, 240, 18, window, (HMENU) VD_MAP_EDITOR_OBJECT_OPTION_PASSABLE_OVER, instance, NULL);
    if (passableOverObjectOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    passableUnderObjectOption = CreateWindow("Button", "Passable (entities appear behind)", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 825, 240, 18, window, (HMENU) VD_MAP_EDITOR_OBJECT_OPTION_PASSABLE_UNDER, instance, NULL);
    if (passableUnderObjectOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    impassableObjectOption = CreateWindow("Button", "Impassable", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 845, 100, 18, window, (HMENU) VD_MAP_EDITOR_OBJECT_OPTION_IMPASSABLE, instance, NULL);
    if (impassableObjectOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    tileSelected = FALSE;

    return window;
};



/* **********************************************************************************************************************
   ProcessMessages
   ---------------
   Processes messages sent and received by and from the Map Editor.
   TODO
   #param
   **********************************************************************************************************************/

void VDMapEditor::ProcessMessages(LPDIRECT3DDEVICE9 direct3DDevice, VDInput* input, vector<VDNPC>* npcs, vector<VDEntity>* objects) {
    // set the input to the Map Editor
    if (input->GetCurrentWindow() != window) {
        input->ChangeInputWindow(window);
    };

    // Load Sprite Sheet button
    ///////////////////////////

    if (SendMessage(loadSpriteButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
        char text[FILE_NAME_MAX_SIZE];

        // open a sprite image
        VDOpenFile(window, "Sprites (*.png,*.bmp)\0*.png;*.bmp\0All Files\0*.*\0", text);

        // make sure the user selected a file
        if (strcmp(text, "") != 0) {
            // set the sprite file location
            (void) strcpy_s(tileSheetImageLocation, text);

            // clear the selected sprite stuff
            (void) strcpy_s(selectedTileImageLocation, "");
            tileSelected = FALSE;

            // add the sprite to the editor so it may be drawn
            RECT drawingArea;
            (void) SetRect(&drawingArea, 0, 0, 160, 160);
            if (tileSheet != NULL) {
                tileSheet->FreeTexture();
                delete tileSheet;
                tileSheet = NULL;
            };
            tileSheet = new VDSprite(direct3DDevice, text, drawingArea, D3DXVECTOR3((float) tileSheetDimensions.left, (float) tileSheetDimensions.top, 0.0f), 0xFF000000);
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // user selects a tile
    //////////////////////

    if (tileSheet != NULL) {
        // get the current cursor position
        POINT mousePosition;
        GetCursorPos(&mousePosition);

        // take a look at the Map Editor window location
        RECT windowDimensions;
        (void) GetWindowRect(window, &windowDimensions);

        // acquire the real mouse position
        POINT mapEditorMousePosition;
        mapEditorMousePosition.x = mousePosition.x - GetSystemMetrics(SM_CXFIXEDFRAME) - windowDimensions.left;
        mapEditorMousePosition.y = mousePosition.y - (GetSystemMetrics(SM_CYFIXEDFRAME) + GetSystemMetrics(SM_CYSMCAPTION) + GetSystemMetrics(SM_CYMENU)) - windowDimensions.top;

        // see if cursor is in the sprite area
        if (mapEditorMousePosition.x >= tileSheetDimensions.left && mapEditorMousePosition.x < tileSheetDimensions.right && mapEditorMousePosition.y >= tileSheetDimensions.top && mapEditorMousePosition.y < tileSheetDimensions.bottom) {
            // user clicks on the sprite sheet
            if (input->GetMouseInput().rgbButtons[0] & 0x80) {
                // let the map editor know we selected a tile
                tileSelected = TRUE;

                // set up the tile source rectangle
                mapEditorMousePosition.x -= tileSheetDimensions.left;
                mapEditorMousePosition.y -= tileSheetDimensions.top;
                LONG tileSelectedX = mapEditorMousePosition.x - (mapEditorMousePosition.x % 32);
                LONG tileSelectedY = mapEditorMousePosition.y - (mapEditorMousePosition.y % 32);

                // get the current source rectangle selection
                RECT currentSelection;
                tileSheet->GetSourceRectangle(&currentSelection);
                currentSelection.left += tileSelectedX;
                currentSelection.top += tileSelectedY;
                currentSelection.right = currentSelection.left + 32;
                currentSelection.bottom = currentSelection.top + 32;

                // make sure we selected a different tile
                if (tileSheetSourceRectangle.left != currentSelection.left || tileSheetSourceRectangle.top != currentSelection.top || tileSheetSourceRectangle.right != currentSelection.right || tileSheetSourceRectangle.bottom != currentSelection.bottom) {
                    // set the new 32x32 source rectangle
                    tileSheetSourceRectangle.left = currentSelection.left;
                    tileSheetSourceRectangle.top = currentSelection.top;
                    tileSheetSourceRectangle.right = currentSelection.right;
                    tileSheetSourceRectangle.bottom = currentSelection.bottom; 

                    // set selected tile to an empty string since we might have to create a new texture
                    (void) strcpy_s(selectedTileImageLocation, "");
                };
            };
        };
    };

    // Up button
    ////////////

    if (SendMessage(shiftTileSpriteUp, BM_GETSTATE, 0, 0) == BST_FOCUS || input->GetKeyboardInput()[DIK_UP] & 0x80) {
        if (tileSheet != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) tileSheet->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            if (drawingArea.top == 0) {
                LONG bottom = tileSheet->GetSpriteHeight();
                tileSheet->SetSourceRectangle(drawingArea.left, bottom - 160, drawingArea.right, bottom);
            } else {
                tileSheet->SetSourceRectangle(drawingArea.left, drawingArea.top - 32, drawingArea.right, drawingArea.bottom - 32);
            };

            // let the map editor know we need to reselect
            (void) strcpy_s(selectedTileImageLocation, "");
            tileSelected = FALSE;
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Down button
    //////////////

    if (SendMessage(shiftTileSpriteDown, BM_GETSTATE, 0, 0) == BST_FOCUS || input->GetKeyboardInput()[DIK_DOWN] & 0x80) {
        if (tileSheet != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) tileSheet->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            LONG bottom = tileSheet->GetSpriteHeight();
            if (drawingArea.bottom == bottom) {
                tileSheet->SetSourceRectangle(drawingArea.left, 0, drawingArea.right, 160);
            } else {
                tileSheet->SetSourceRectangle(drawingArea.left, drawingArea.top + 32, drawingArea.right, drawingArea.bottom + 32);
            };

            // let the map editor know we need to reselect
            (void) strcpy_s(selectedTileImageLocation, "");
            tileSelected = FALSE;
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Left button
    //////////////

    if (SendMessage(shiftTileSpriteLeft, BM_GETSTATE, 0, 0) == BST_FOCUS || input->GetKeyboardInput()[DIK_LEFT] & 0x80) {
        if (tileSheet != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) tileSheet->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            if (drawingArea.left == 0) {
                LONG right = tileSheet->GetSpriteWidth();
                tileSheet->SetSourceRectangle(right - 160, drawingArea.top, right, drawingArea.bottom);
            } else {
                tileSheet->SetSourceRectangle(drawingArea.left - 32, drawingArea.top, drawingArea.right - 32, drawingArea.bottom);
            };

            // let the map editor know we need to reselect
            (void) strcpy_s(selectedTileImageLocation, "");
            tileSelected = FALSE;
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Right button
    ///////////////

    if (SendMessage(shiftTileSpriteRight, BM_GETSTATE, 0, 0) == BST_FOCUS || input->GetKeyboardInput()[DIK_RIGHT] & 0x80) {
        if (tileSheet != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) tileSheet->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            LONG right = tileSheet->GetSpriteWidth();
            if (drawingArea.right == right) {
                tileSheet->SetSourceRectangle(0, drawingArea.top, 160, drawingArea.bottom);
            } else {
                tileSheet->SetSourceRectangle(drawingArea.left + 32, drawingArea.top, drawingArea.right + 32, drawingArea.bottom);
            };

            // let the map editor know we need to reselect
            (void) strcpy_s(selectedTileImageLocation, "");
            tileSelected = FALSE;
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Load New Area button
    ///////////////////////

    if (SendMessage(loadNewAreaButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
        char text[FILE_NAME_MAX_SIZE];

        // open the new world file
        VDOpenFile(window, "VDEngine Maps\0*.vdm\0", text);

        // make sure the user selected a file
        if (strcmp(text, "") != 0) {
            // set the new area location
            (void) strcpy_s(newAreaFileLocation, text);

            // remove the path name
            char actualName[FILE_NAME_MAX_SIZE];
            int n = 0, m = 0, p = 0, numSlashes = 0;
            while (text[n] != '\0') {
                if (text[n] == '\\') {
                    numSlashes++;
                };
                n++;
            };
            n = 0;
            while (text[n] != '\0') {
                if (text[n] == '\\') {
                    m++;
                };
                n++;
                if (text[n] != '\0' && m == numSlashes) {
                    actualName[p] = text[n];
                    p++;
                };
            };
            actualName[p] = '\0';

            // display the new world file
            SetWindowText(newAreaStatic, actualName);
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Tile radio buttons
    /////////////////////

    // Remove
    if (SendMessage(nullTileOption, BM_GETCHECK, 0, 0) == BST_CHECKED) {
        if (selectedRadioButton != 0) {
            selectedRadioButton = 0;

            // clear the object list & sprite
            (void) SendMessage(objectListBox, LB_RESETCONTENT, 0, 0);
            (void) strcpy_s(previousObjectSelection, "");
            if (objectSprite != NULL) {
                objectSprite->FreeTexture();
                delete objectSprite;
                objectSprite = NULL;
            };
        };
    };
    // Passable (Entities On Top)
    if (SendMessage(passableOverTileOption, BM_GETCHECK, 0, 0) == BST_CHECKED) {
        if (selectedRadioButton != 1) {
            selectedRadioButton = 1;

            // clear the object list & sprite
            (void) SendMessage(objectListBox, LB_RESETCONTENT, 0, 0);
            (void) strcpy_s(previousObjectSelection, "");
            if (objectSprite != NULL) {
                objectSprite->FreeTexture();
                delete objectSprite;
                objectSprite = NULL;
            };
        };
    };
    // Passable (Entities Hidden)
    if (SendMessage(passableUnderTileOption, BM_GETCHECK, 0, 0) == BST_CHECKED) {
        if (selectedRadioButton != 2) {
            selectedRadioButton = 2;

            // clear the object list & sprite
            (void) SendMessage(objectListBox, LB_RESETCONTENT, 0, 0);
            (void) strcpy_s(previousObjectSelection, "");
            if (objectSprite != NULL) {
                objectSprite->FreeTexture();
                delete objectSprite;
                objectSprite = NULL;
            };
        };
    };
    // Impassable
    if (SendMessage(impassableTileOption, BM_GETCHECK, 0, 0) == BST_CHECKED) {
        if (selectedRadioButton != 3) {
            selectedRadioButton = 3;

            // clear the object list & sprite
            (void) SendMessage(objectListBox, LB_RESETCONTENT, 0, 0);
            (void) strcpy_s(previousObjectSelection, "");
            if (objectSprite != NULL) {
                objectSprite->FreeTexture();
                delete objectSprite;
                objectSprite = NULL;
            };
        };
    };
    // New Area
    if (SendMessage(newAreaTileOption, BM_GETCHECK, 0, 0) == BST_CHECKED) {
        if (selectedRadioButton != 4) {
            selectedRadioButton = 4;

            // clear the object list & sprite
            (void) SendMessage(objectListBox, LB_RESETCONTENT, 0, 0);
            (void) strcpy_s(previousObjectSelection, "");
            if (objectSprite != NULL) {
                objectSprite->FreeTexture();
                delete objectSprite;
                objectSprite = NULL;
            };
        };
    };
    // Remove
    if (SendMessage(nullObjectOption, BM_GETCHECK, 0, 0) == BST_CHECKED) {
        if (selectedRadioButton != 5) {
            selectedRadioButton = 5;

            // clear the object list & sprite
            (void) SendMessage(objectListBox, LB_RESETCONTENT, 0, 0);
            (void) strcpy_s(previousObjectSelection, "");
            if (objectSprite != NULL) {
                objectSprite->FreeTexture();
                delete objectSprite;
                objectSprite = NULL;
            };
        };
    };
    // NPC
    if (SendMessage(npcOption, BM_GETCHECK, 0, 0) == BST_CHECKED) {
        if (selectedRadioButton != 6) {
            selectedRadioButton = 6;

            // clear the object list & sprite
            (void) SendMessage(objectListBox, LB_RESETCONTENT, 0, 0);
            (void) strcpy_s(previousObjectSelection, "");
            if (objectSprite != NULL) {
                objectSprite->FreeTexture();
                delete objectSprite;
                objectSprite = NULL;
            };

            // populate the object list with NPCs
            for (unsigned int n = 0; n < npcs->size(); n++) {
                (void) SendMessage(objectListBox, LB_ADDSTRING, 0, (LPARAM) npcs->at(n).GetName());
            };
        };
    };
    // Passable (entities appear on top)
    if (SendMessage(passableOverObjectOption, BM_GETCHECK, 0, 0) == BST_CHECKED) {
        if (selectedRadioButton != 7) {
            selectedRadioButton = 7;

            // clear the object list & sprite
            (void) SendMessage(objectListBox, LB_RESETCONTENT, 0, 0);
            (void) strcpy_s(previousObjectSelection, "");
            if (objectSprite != NULL) {
                objectSprite->FreeTexture();
                delete objectSprite;
                objectSprite = NULL;
            };

            // populate the object list with objects
            for (unsigned int n = 0; n < objects->size(); n++) {
                (void) SendMessage(objectListBox, LB_ADDSTRING, 0, (LPARAM) objects->at(n).GetName());
            };
        };
    };
    // Passable (entities appear behind)
    if (SendMessage(passableUnderObjectOption, BM_GETCHECK, 0, 0) == BST_CHECKED) {
        if (selectedRadioButton != 8) {
            selectedRadioButton = 8;

            // clear the object list & sprite
            (void) SendMessage(objectListBox, LB_RESETCONTENT, 0, 0);
            (void) strcpy_s(previousObjectSelection, "");
            if (objectSprite != NULL) {
                objectSprite->FreeTexture();
                delete objectSprite;
                objectSprite = NULL;
            };

            // populate the object list with objects
            for (unsigned int n = 0; n < objects->size(); n++) {
                (void) SendMessage(objectListBox, LB_ADDSTRING, 0, (LPARAM) objects->at(n).GetName());
            };
        };
    };
    // Impassable
    if (SendMessage(impassableObjectOption, BM_GETCHECK, 0, 0) == BST_CHECKED) {
        if (selectedRadioButton != 9) {
            selectedRadioButton = 9;

            // clear the object list & sprite
            (void) SendMessage(objectListBox, LB_RESETCONTENT, 0, 0);
            (void) strcpy_s(previousObjectSelection, "");
            if (objectSprite != NULL) {
                objectSprite->FreeTexture();
                delete objectSprite;
                objectSprite = NULL;
            };

            // populate the object list with objects
            for (unsigned int n = 0; n < objects->size(); n++) {
                (void) SendMessage(objectListBox, LB_ADDSTRING, 0, (LPARAM) objects->at(n).GetName());
            };
        };
    };

    // user has an object selected
    //////////////////////////////

    int currentObjectIndex = (int) SendMessage(objectListBox, LB_GETCURSEL, 0, 0);
    if (currentObjectIndex != LB_ERR) {
        // get the name of the current selected object
        char currentObjectSelection[FILE_NAME_MAX_SIZE];
        (void) SendMessage(objectListBox, LB_GETTEXT, (WPARAM) currentObjectIndex, (LPARAM) &currentObjectSelection);

        // < button
        ///////////

        if (SendMessage(previousAnimationButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
            if (objectSprite != NULL) {
                // get the sprite's source rectangle
                RECT drawingArea;
                (void) objectSprite->GetSourceRectangle(&drawingArea);

                // see if we're wrapping around
                if (drawingArea.left == 0) {
                    LONG right = objectSprite->GetSpriteWidth();
                    objectSprite->SetSourceRectangle(right - 32, drawingArea.top, right, drawingArea.bottom);
                } else {
                    objectSprite->SetSourceRectangle(drawingArea.left - 32, drawingArea.top, drawingArea.right - 32, drawingArea.bottom);
                };
            };

            // make sure we're not still focused on the button
            (void) SetFocus(window);
        };

        // > button
        ///////////

        if (SendMessage(nextAnimationButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
            if (objectSprite != NULL) {
                // get the sprite's source rectangle
                RECT drawingArea;
                (void) objectSprite->GetSourceRectangle(&drawingArea);

                // see if we're wrapping around
                LONG right = objectSprite->GetSpriteWidth();
                if (drawingArea.right == right) {
                    objectSprite->SetSourceRectangle(0, drawingArea.top, 32, drawingArea.bottom);
                } else {
                    objectSprite->SetSourceRectangle(drawingArea.left + 32, drawingArea.top, drawingArea.right + 32, drawingArea.bottom);
                };
            };

            // make sure we're not still focused on the button
            (void) SetFocus(window);
        };

        // user selects a different object
        //////////////////////////////////

        if (strcmp(previousObjectSelection, currentObjectSelection) != 0) {
            // copy the current selection to previous
            (void) strcpy_s(previousObjectSelection, currentObjectSelection);

            // make sure the editor knows the file location
            if (selectedRadioButton == 6) {
                for (unsigned int n = 0; n < npcs->size(); n++) {
                    if (strcmp(currentObjectSelection, npcs->at(n).GetName()) == 0) {
                        (void) strcpy_s(selectedObjectImageLocation, npcs->at(n).GetImageFileLocation());
                    };
                };
            } else if (selectedRadioButton >= 7 && selectedRadioButton <= 9) {
                for (unsigned int n = 0; n < objects->size(); n++) {
                    if (strcmp(currentObjectSelection, objects->at(n).GetName()) == 0) {
                        (void) strcpy_s(selectedObjectImageLocation, objects->at(n).GetImageFileLocation());
                    };
                };
            };

            // update the window
            RECT drawingArea;
            (void) SetRect(&drawingArea, 0, 0, 32, 32);

            // load the NPC sprite
            if (objectSprite != NULL) {
                objectSprite->FreeTexture();
                delete objectSprite;
                objectSprite = NULL;
            };
            objectSprite = new VDSprite(direct3DDevice, selectedObjectImageLocation, drawingArea, D3DXVECTOR3((float) objectSpriteDimensions.left, (float) objectSpriteDimensions.top, 0.0f), 0xFF000000);
        };
    };
};



// TODO

void VDMapEditor::Clear() {
    (void) SetWindowText(newAreaStatic, "");
    (void) SetWindowText(newAreaStartingPointX, "0");
    (void) SetWindowText(newAreaStartingPointY, "0");

    if (tileSheet != NULL) {
        tileSheet->FreeTexture();
        delete tileSheet;
        tileSheet = NULL;
    };
    if (objectSprite != NULL) {
        objectSprite->FreeTexture();
        delete objectSprite;
        objectSprite = NULL;
    };
    tileSelected = FALSE;
    tileSheetSourceRectangle.left = 0;
    tileSheetSourceRectangle.top = 0;
    tileSheetSourceRectangle.right = 32;
    tileSheetSourceRectangle.bottom = 32;

    (void) strcpy_s(tileSheetImageLocation, "");
    (void) strcpy_s(selectedTileImageLocation, "");
    (void) strcpy_s(newAreaFileLocation, "");
    (void) strcpy_s(selectedObjectImageLocation, "");
    (void) strcpy_s(previousObjectSelection, "");
};



/* **********************************************************************************************************************
   CreateOrLoadTexture
   -------------------
   Loads a tile represented on the tile sheet if it exists, creates it otherwise.
   **********************************************************************************************************************/

void VDMapEditor::CreateOrLoadTexture(LPDIRECT3DDEVICE9 direct3DDevice) {
    char newName[FILE_NAME_MAX_SIZE], tileSheetName[FILE_NAME_MAX_SIZE], extension[10];
    LPDIRECT3DTEXTURE9 tempTexture = NULL;

    // remove the path name and extension from the tile sheet location
    int n = 0, m = 0, p = 0, numSlashes = 0;
    while (tileSheetImageLocation[n] != '\0') {
        if (tileSheetImageLocation[n] == '\\') {
            numSlashes++;
        };
        n++;
    };
    n = 0;
    while (tileSheetImageLocation[n] != '.') {
        if (tileSheetImageLocation[n] == '\\') {
            m++;
        };
        n++;
        if (tileSheetImageLocation[n] != '.' && m == numSlashes) {
            tileSheetName[p] = tileSheetImageLocation[n];
            p++;
        };
    };
    tileSheetName[p] = '\0';
    m = 0;
    while (tileSheetImageLocation[n] != '\0') {
        extension[m] = tileSheetImageLocation[n];
        n++;
        m++;
    };
    extension[m] = '\0';
    (void) strcpy_s(newName, "Images/");
    (void) strcat_s(newName, tileSheetName);

    // now add the X,Y loc and file extension
    (void) strcat_s(newName, "-");
    char loc[10];
    (void) _itoa_s(tileSheetSourceRectangle.left, loc, sizeof(loc));
    (void) strcat_s(newName, loc);
    (void) strcat_s(newName, ",");
    (void) strcpy_s(loc, "");
    (void) _itoa_s(tileSheetSourceRectangle.top, loc, sizeof(loc));
    (void) strcat_s(newName, loc);
    (void) strcat_s(newName, extension);

    // see if the texture exists already
    HRESULT returnValue = D3DXCreateTextureFromFileEx(direct3DDevice, newName, D3DX_DEFAULT_NONPOW2, D3DX_DEFAULT_NONPOW2, D3DX_DEFAULT, NULL, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, 0xFFFFFFFF, NULL, NULL, &tempTexture);
    if (FAILED(returnValue)) {
        // create a new texture
        returnValue = D3DXCreateTexture(direct3DDevice, 32, 32, D3DX_DEFAULT, 0, D3DFMT_X8R8G8B8, D3DPOOL_SCRATCH, &tempTexture);

        // lock the textures
        D3DLOCKED_RECT sourceSprite, destSprite;
        returnValue = tileSheet->GetTexture()->LockRect(0, &sourceSprite, NULL, D3DLOCK_READONLY);
        if (FAILED(returnValue)) {
	        VDDirectXError("Failed to lock the texture.", returnValue, __FILE__, __LINE__);
        }
        returnValue = tempTexture->LockRect(0, &destSprite, NULL, D3DLOCK_DISCARD);
        if (FAILED(returnValue)) {
            tileSheet->GetTexture()->UnlockRect(0);
            VDDirectXError("Failed to lock the texture.", returnValue, __FILE__, __LINE__);
        };

        // copy the data from the start of the source rectangle in the texture to the end
        UINT sourceSpriteWidth = tileSheet->GetSpriteWidth();
        UINT bytesPerPixel = 4;
        const BYTE* sourceLoc = (const BYTE*) sourceSprite.pBits + tileSheetSourceRectangle.top * sourceSpriteWidth * bytesPerPixel
                                                                 + tileSheetSourceRectangle.left * bytesPerPixel;
        for (int n = 0; n < 32; n++) {
            BYTE* destAddress = (BYTE *) destSprite.pBits + n * destSprite.Pitch;
            memcpy(destAddress, sourceLoc, 32 * bytesPerPixel);
            sourceLoc += sourceSprite.Pitch;
        };

        // unlock the textures
        tileSheet->GetTexture()->UnlockRect(0);
        tempTexture->UnlockRect(0);

        // save the texture to hard disk - requires a directory Images to be in the app directory
        if (strcmp(extension, ".png") == 0) {
            returnValue = D3DXSaveTextureToFile(newName, D3DXIFF_PNG, tempTexture, NULL);
            if (FAILED(returnValue)) {
                VDDirectXError("Failed to save texture.", returnValue, __FILE__, __LINE__);
            };
        } else if (strcmp(extension, ".bmp") == 0) {
            returnValue = D3DXSaveTextureToFile(newName, D3DXIFF_BMP, tempTexture, NULL);
            if (FAILED(returnValue)) {
                VDDirectXError("Failed to save texture.", returnValue, __FILE__, __LINE__);
            };
        };
    };

    // free memory
    if (tempTexture != NULL) {
        tempTexture->Release();
        tempTexture = NULL;
    };

    (void) strcpy_s(selectedTileImageLocation, newName);
};