/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "VDNPCEditor.h"



/* **********************************************************************************************************************
   Constructor
   **********************************************************************************************************************/

VDNPCEditor::VDNPCEditor() {
    window = NULL;
    npcListBox = NULL;
    (void) strcpy_s(currentNPCSelection, "");
    nameEditBox = NULL;
    loadSpriteButton = NULL;
    npcSpriteSheet = NULL;
    (void) strcpy_s(npcSpriteSheetLocation, "");
    npcSprite = NULL;
    (void) strcpy_s(npcSpriteImageLocation, "");
    (void) SecureZeroMemory(&npcSpriteDimensions, sizeof(npcSpriteDimensions));
    shiftNPCUp = NULL;
    shiftNPCDown = NULL;
    previousAnimation = NULL;
    nextAnimation = NULL;
    movementSpeedEditBox = NULL;
    removeNPCButton = NULL;
    newNPCButton = NULL;
    saveNPCButton = NULL;
    dialogueListBox = NULL;
    (void) strcpy_s(currentDialogueSelection, "");
    textEditBox = NULL;
    textIDEditBox = NULL;
    removeDialogueButton = NULL;
    saveDialogueButton = NULL;
    optionListBox = NULL;
    (void) strcpy_s(currentOptionSelection, "");
    textEditBox2 = NULL;
    textIDEditBox2 = NULL;
    removeOptionButton = NULL;
    saveOptionButton = NULL;
    saveChangesButton = NULL;
    reloadNPCs = FALSE;
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

VDNPCEditor::~VDNPCEditor() {
    FreeMemory();
};



// TODO

void VDNPCEditor::FreeMemory() {
    if (npcSprite != NULL) {
        npcSprite->FreeTexture();
        delete npcSprite;
        npcSprite = NULL;
    };
    if (npcSpriteSheet != NULL) {
        npcSpriteSheet->FreeTexture();
        delete npcSpriteSheet;
        npcSpriteSheet = NULL;
    };
};



/* **********************************************************************************************************************
   Load
   ----
   Creates a new NPC Editor window.
   #param  instance      The instance of the application.
   #param  parentWindow  Handle to the main window (to use for the starting x, y position)
   #param  className     The name of the window class.
   #return  Handle to the window.
   **********************************************************************************************************************/

HWND VDNPCEditor::Load(HINSTANCE instance, HWND parentWindow, LPCSTR className) {
    RECT windowDimensions;
    (void) GetWindowRect(parentWindow, &windowDimensions);
    window = CreateWindowEx(WS_EX_TOPMOST | WS_EX_TOOLWINDOW,      // extended window styles
                            className,                             // class name
                            "NPC Editor",                          // window name
                            WS_CAPTION | WS_SYSMENU | WS_VISIBLE,  // window styles
                            windowDimensions.left + 100,           // x coordinate (0 = left)
                            windowDimensions.top + 100,            // y coordinate (0 = top)
                            440,                                   // width
                            530,                                   // height
                            NULL,                                  // parent/owner window
                            NULL,                                  // menu
                            instance,                              // instance
                            NULL);                                 // used for CREATESTRUCT structure
    if (window == NULL) {
        FreeMemory();
        VDError("Failed to create the window.", __FILE__, __LINE__);
    };

    // NPC's
    HWND npcListStatic = CreateWindow("Static", "NPC's:", WS_CHILD | WS_VISIBLE, 10, 10, 40, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_NPC_LIST, instance, NULL);
    if (npcListStatic == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    npcListBox = CreateWindow("ListBox", NULL, WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL, 10, 30, 100, 100, window, (HMENU) VD_NPC_EDITOR_NPC_LIST, instance, NULL);
    if (npcListBox == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    (void) strcpy_s(currentNPCSelection, "");
    HWND npcNameStatic = CreateWindow("Static", "Name:", WS_CHILD | WS_VISIBLE, 120, 30, 50, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_NAME, instance, NULL);
    if (npcNameStatic == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    nameEditBox = CreateWindow("Edit", "", WS_CHILD | WS_BORDER | WS_VISIBLE, 180, 30, 140, 18, window, (HMENU) VD_NPC_EDITOR_NAME, instance, NULL);
    if (nameEditBox == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    loadSpriteButton = CreateWindow("Button", "Load Sprite", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 120, 60, 90, 20, window, (HMENU) VD_NPC_EDITOR_LOAD_SPRITE, instance, NULL);
    if (loadSpriteButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND spriteBorderStatic = CreateWindow("Static", "", WS_CHILD | WS_VISIBLE | WS_BORDER, 120, 85, 48, 48, window, (HMENU) VD_NPC_EDITOR_STATIC_SPRITE_BORDER, instance, NULL);
    if (spriteBorderStatic == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    FreeMemory();
    (void) strcpy_s(npcSpriteSheetLocation, "");
    (void) strcpy_s(npcSpriteImageLocation, "");
    npcSpriteDimensions.left = 128;
    npcSpriteDimensions.top = 93;
    npcSpriteDimensions.right = 160;
    npcSpriteDimensions.bottom = 125;
    shiftNPCUp = CreateWindow("Button", "/\\", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 168, 85, 20, 20, window, (HMENU) VD_NPC_EDITOR_SHIFT_NPC_UP, instance, NULL);
    if (shiftNPCUp == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    shiftNPCDown = CreateWindow("Button", "\\/", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 168, 113, 20, 20, window, (HMENU) VD_NPC_EDITOR_SHIFT_NPC_DOWN, instance, NULL);
    if (shiftNPCDown == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    previousAnimation = CreateWindow("Button", "<", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 120, 133, 20, 20, window, (HMENU) VD_NPC_EDITOR_PREVIOUS_ANIMATION, instance, NULL);
    if (previousAnimation == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    nextAnimation = CreateWindow("Button", ">", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 148, 133, 20, 20, window, (HMENU) VD_NPC_EDITOR_NEXT_ANIMATION, instance, NULL);
    if (nextAnimation == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND movementSpeedStatic = CreateWindow("Static", "Movement Speed:", WS_CHILD | WS_VISIBLE, 220, 60, 120, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_MOVEMENT_SPEED, instance, NULL);
    if (movementSpeedStatic == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    movementSpeedEditBox = CreateWindow("Edit", "3", WS_CHILD | WS_VISIBLE | WS_BORDER, 350, 60, 30, 18, window, (HMENU) VD_NPC_EDITOR_MOVEMENT_SPEED, instance, NULL);
    if (movementSpeedEditBox == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    removeNPCButton = CreateWindow("Button", "Remove NPC", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 10, 133, 100, 20, window, (HMENU) VD_NPC_EDITOR_REMOVE_NPC, instance, NULL);
    if (removeNPCButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    newNPCButton = CreateWindow("Button", "New NPC", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 330, 20, 70, 20, window, (HMENU) VD_NPC_EDITOR_NEW_NPC, instance, NULL);
    if (newNPCButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    saveNPCButton = CreateWindow("Button", "Save NPC", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 220, 100, 80, 20, window, (HMENU) VD_NPC_EDITOR_SAVE_NPC, instance, NULL);
    if (saveNPCButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    // NPC dialogue
    HWND dialogueStatic = CreateWindow("Static", "Dialogues", WS_CHILD | WS_VISIBLE, 10, 160, 70, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_DIALOGUE, instance, NULL);
    if (dialogueStatic == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    dialogueListBox = CreateWindow("ListBox", "", WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL, 10, 180, 200, 110, window, (HMENU) VD_NPC_EDITOR_DIALOGUE, instance, NULL);
    if (dialogueListBox == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    (void) strcpy_s(currentDialogueSelection, "");
    HWND textStatic = CreateWindow("Static", "Text:", WS_CHILD | WS_VISIBLE, 220, 160, 50, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_TEXT, instance, NULL);
    if (textStatic == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    textEditBox = CreateWindow("Edit", "", WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL | ES_MULTILINE, 220, 180, 200, 60, window, (HMENU) VD_NPC_EDITOR_TEXT, instance, NULL);
    if (textEditBox == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND textIDStatic = CreateWindow("Static", "Text ID:", WS_CHILD | WS_VISIBLE, 220, 250, 50, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_TEXT_ID, instance, NULL);
    if (textIDStatic == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    textIDEditBox = CreateWindow("Edit", "", WS_CHILD | WS_BORDER | WS_VISIBLE, 275, 250, 30, 18, window, (HMENU) VD_NPC_EDITOR_TEXT_ID, instance, NULL);
    if (textIDEditBox == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    removeDialogueButton = CreateWindow("Button", "Remove Dialogue", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 10, 283, 130, 20, window, (HMENU) VD_NPC_EDITOR_REMOVE_DIALOGUE, instance, NULL);
    if (removeDialogueButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    saveDialogueButton = CreateWindow("Button", "Save Text", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 320, 250, 80, 20, window, (HMENU) VD_NPC_EDITOR_SAVE_DIALOGUE, instance, NULL);
    if (saveDialogueButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    // options
    HWND optionsStatic = CreateWindow("Static", "User Response Options", WS_CHILD | WS_VISIBLE, 10, 310, 200, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_OPTIONS, instance, NULL);
    if (optionsStatic == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    optionListBox = CreateWindow("ListBox", "", WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL, 10, 330, 200, 110, window, (HMENU) VD_NPC_EDITOR_OPTIONS, instance, NULL);
    if (optionListBox == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    (void) strcpy_s(currentOptionSelection, "");
    HWND textStatic2 = CreateWindow("Static", "Text:", WS_CHILD | WS_VISIBLE, 220, 310, 70, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_TEXT2, instance, NULL);
    if (textStatic2 == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    textEditBox2 = CreateWindow("Edit", "", WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL | ES_MULTILINE, 220, 330, 200, 60, window, (HMENU) VD_NPC_EDITOR_TEXT2, instance, NULL);
    if (textEditBox2 == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND textIDStatic2 = CreateWindow("Static", "Next ID:", WS_CHILD | WS_VISIBLE, 220, 400, 55, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_TEXT_ID2, instance, NULL);
    if (textIDStatic2 == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    textIDEditBox2 = CreateWindow("Edit", "", WS_CHILD | WS_VISIBLE | WS_BORDER, 278, 400, 30, 18, window, (HMENU) VD_NPC_EDITOR_TEXT_ID2, instance, NULL);
    if (textIDEditBox2 == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    removeOptionButton = CreateWindow("Button", "Remove Response", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 10, 432, 140, 20, window, (HMENU) VD_NPC_EDITOR_REMOVE_OPTION, instance, NULL);
    if (removeOptionButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    saveOptionButton = CreateWindow("Button", "Save Text", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 320, 400, 80, 20, window, (HMENU) VD_NPC_EDITOR_SAVE_OPTION, instance, NULL);
    if (saveOptionButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    // save NPC's
    saveChangesButton = CreateWindow("Button", "SAVE CHANGES", WS_CHILD | BS_PUSHBUTTON | WS_BORDER | WS_VISIBLE, 115, 460, 200, 40, window, (HMENU) VD_NPC_EDITOR_SAVE_CHANGES, instance, NULL);
    if (saveChangesButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    return window;
};



/* **********************************************************************************************************************
   ProcessMessages
   ---------------
   Processes messages sent by and received from the NPC Editor.
   #param TODO
   **********************************************************************************************************************/

void VDNPCEditor::ProcessMessages(LPDIRECT3DDEVICE9 direct3DDevice, VDInput *input, vector<VDNPC>* npcs) {
    // set the input to the NPC Editor
    if (input->GetCurrentWindow() != window) {
        input->ChangeInputWindow(window);
    };

    // populate the npc list if it's empty
    if (SendMessage(npcListBox, LB_GETCOUNT, 0, 0) == 0) {
        for (size_t n = 0; n < npcs->size(); n++) {
            (void) SendMessage(npcListBox, LB_ADDSTRING, 0, (LPARAM) npcs->at(n).GetName());
        };
    };

    // Save Changes button
    //////////////////////

    if (SendMessage(saveChangesButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
        // save the NPCs
        WriteNPCs(npcs);

        // lets other editors know we've changed NPC's
        reloadNPCs = TRUE;

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Load Sprite button
    /////////////////////

    if (SendMessage(loadSpriteButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
        char text[FILE_NAME_MAX_SIZE];

        // open a sprite image
        VDOpenFile(window, "Sprites (*.png,*.bmp)\0*.png;*.bmp\0All Files\0*.*\0", text);

        // make sure the user selected a file
        if (strcmp(text, "") != 0) {
            // set the sprite file location
            (void) strcpy_s(npcSpriteSheetLocation, text);

            // add the sprite to the editor so that it may be drawn
            RECT drawingArea;
            (void) SetRect(&drawingArea, 0, 0, 32, 32);
            FreeMemory();
            npcSpriteSheet = new VDSprite(direct3DDevice, text, drawingArea, D3DXVECTOR3((float) npcSpriteDimensions.left, (float) npcSpriteDimensions.top, 0.0f), 0xFF000000);
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Up button
    ////////////

    if (SendMessage(shiftNPCUp, BM_GETSTATE, 0, 0) == BST_FOCUS  || input->GetKeyboardInput()[DIK_UP] & 0x80) {
        if (npcSpriteSheet != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) npcSpriteSheet->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            if (drawingArea.top == 0) {
                LONG bottom = npcSpriteSheet->GetSpriteHeight();
                npcSpriteSheet->SetSourceRectangle(drawingArea.left, bottom - 32, drawingArea.right, bottom);
            } else {
                npcSpriteSheet->SetSourceRectangle(drawingArea.left, drawingArea.top - 32, drawingArea.right, drawingArea.bottom - 32);
            };
        };
        if (npcSprite != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) npcSprite->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            if (drawingArea.top == 0) {
                LONG bottom = npcSprite->GetSpriteHeight();
                npcSprite->SetSourceRectangle(drawingArea.left, bottom - 32, drawingArea.right, bottom);
            } else {
                npcSprite->SetSourceRectangle(drawingArea.left, drawingArea.top - 32, drawingArea.right, drawingArea.bottom - 32);
            };
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Down button
    //////////////

    if (SendMessage(shiftNPCDown, BM_GETSTATE, 0, 0) == BST_FOCUS || input->GetKeyboardInput()[DIK_DOWN] & 0x80) {
        if (npcSpriteSheet != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) npcSpriteSheet->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            LONG bottom = npcSpriteSheet->GetSpriteHeight();
            if (drawingArea.bottom == bottom) {
                npcSpriteSheet->SetSourceRectangle(drawingArea.left, 0, drawingArea.right, 32);
            } else {
                npcSpriteSheet->SetSourceRectangle(drawingArea.left, drawingArea.top + 32, drawingArea.right, drawingArea.bottom + 32);
            };
        };
        if (npcSprite != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) npcSprite->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            LONG bottom = npcSprite->GetSpriteHeight();
            if (drawingArea.bottom == bottom) {
                npcSprite->SetSourceRectangle(drawingArea.left, 0, drawingArea.right, 32);
            } else {
                npcSprite->SetSourceRectangle(drawingArea.left, drawingArea.top + 32, drawingArea.right, drawingArea.bottom + 32);
            };
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Left button
    //////////////

    if (SendMessage(previousAnimation, BM_GETSTATE, 0, 0) == BST_FOCUS || input->GetKeyboardInput()[DIK_LEFT] & 0x80) {
        if (npcSpriteSheet != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) npcSpriteSheet->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            if (drawingArea.left == 0) {
                LONG right = npcSpriteSheet->GetSpriteWidth();
                npcSpriteSheet->SetSourceRectangle(right - 32, drawingArea.top, right, drawingArea.bottom);
            } else {
                npcSpriteSheet->SetSourceRectangle(drawingArea.left - 32, drawingArea.top, drawingArea.right - 32, drawingArea.bottom);
            };
        };
        if (npcSprite != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) npcSprite->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            if (drawingArea.left == 0) {
                LONG right = npcSprite->GetSpriteWidth();
                npcSprite->SetSourceRectangle(right - 32, drawingArea.top, right, drawingArea.bottom);
            } else {
                npcSprite->SetSourceRectangle(drawingArea.left - 32, drawingArea.top, drawingArea.right - 32, drawingArea.bottom);
            };
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Right button
    ///////////////

    if (SendMessage(nextAnimation, BM_GETSTATE, 0, 0) == BST_FOCUS || input->GetKeyboardInput()[DIK_RIGHT] & 0x80) {
        if (npcSpriteSheet != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) npcSpriteSheet->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            LONG right = npcSpriteSheet->GetSpriteWidth();
            if (drawingArea.right == right) {
                npcSpriteSheet->SetSourceRectangle(0, drawingArea.top, 32, drawingArea.bottom);
            } else {
                npcSpriteSheet->SetSourceRectangle(drawingArea.left + 32, drawingArea.top, drawingArea.right + 32, drawingArea.bottom);
            };
        };
        if (npcSprite != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) npcSprite->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            LONG right = npcSprite->GetSpriteWidth();
            if (drawingArea.right == right) {
                npcSprite->SetSourceRectangle(0, drawingArea.top, 32, drawingArea.bottom);
            } else {
                npcSprite->SetSourceRectangle(drawingArea.left + 32, drawingArea.top, drawingArea.right + 32, drawingArea.bottom);
            };
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // New NPC button
    /////////////////

    if (SendMessage(newNPCButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
        // reset the window
        (void) SendMessage(npcListBox, LB_SETCURSEL, (WPARAM) - 1, 0);
        (void) SetWindowText(nameEditBox, "");
        FreeMemory();
        (void) SendMessage(dialogueListBox, LB_RESETCONTENT, 0, 0);
        (void) SetWindowText(textEditBox, "");
        (void) SetWindowText(textIDEditBox, "");
        (void) SendMessage(optionListBox, LB_RESETCONTENT, 0, 0);
        (void) SetWindowText(textEditBox2, "");
        (void) SetWindowText(textIDEditBox2, "");
        (void) strcpy_s(currentNPCSelection, "");
        (void) strcpy_s(currentDialogueSelection, "");
        (void) strcpy_s(currentOptionSelection, "");

        // make sure we're not still focused on the button
        (void) SetFocus(npcListBox);
    };

    // Save NPC button
    //////////////////

    if (SendMessage(saveNPCButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
        char selectedNPC[FILE_NAME_MAX_SIZE], npcName[FILE_NAME_MAX_SIZE];

        // first see if the user has an NPC selected
        int currentNPCIndex = (int) SendMessage(npcListBox, LB_GETCURSEL, 0, 0);
        if (currentNPCIndex != LB_ERR) {
            // get the name of the current selected NPC
            (void) SendMessage(npcListBox, LB_GETTEXT, (WPARAM) currentNPCIndex, (LPARAM) &selectedNPC);
        } else {
            (void) strcpy_s(selectedNPC, "");
        };

        // check for a name, ID, and sprite
        (void) GetWindowText(nameEditBox, npcName, sizeof(npcName));
        if ((strcmp(npcName, "") != 0 && npcSpriteSheet != NULL) || (strcmp(selectedNPC, "") != 0 && npcSprite != NULL)) {
            char newImageFile[FILE_NAME_MAX_SIZE], extension[10];

            // remove the path name, save the extension
            int n = 0, m = 0;
            while (npcSpriteSheetLocation[n] != '.') {
                n++;
            };
            while (npcSpriteSheetLocation[n] != '\0') {
                extension[m] = npcSpriteSheetLocation[n];
                n++;
                m++;
            };
            extension[m] = '\0';

            // use the NPC name for the new texture
            (void) strcpy_s(newImageFile, "Images/");
            (void) strcat_s(newImageFile, npcName);
            (void) strcat_s(newImageFile, extension);

            // create the texture
            if (npcSpriteSheet != NULL && npcSprite == NULL) {
                CreateNPCTexture(direct3DDevice, npcSpriteSheet, newImageFile);
            } else {
                CreateNPCTexture(direct3DDevice, npcSprite, newImageFile);
            };

            // get the movement speed
            char movementSpeed[10];
            (void) GetWindowText(movementSpeedEditBox, movementSpeed, sizeof(movementSpeed));

            // TODO: check if atoi doesn't return the right value and present MessageBox error for user

            // add the NPC
            (void) AddNPC(npcs, selectedNPC, npcName, newImageFile, atoi(movementSpeed));

            // reset the window
            (void) SendMessage(npcListBox, LB_RESETCONTENT, 0, 0);
            (void) SetWindowText(nameEditBox, "");
            FreeMemory();
            (void) SetWindowText(movementSpeedEditBox, "3");
            (void) SendMessage(dialogueListBox, LB_RESETCONTENT, 0, 0);
            (void) SetWindowText(textEditBox, "");
            (void) SetWindowText(textIDEditBox, "");
            (void) SendMessage(optionListBox, LB_RESETCONTENT, 0, 0);
            (void) SetWindowText(textEditBox2, "");
            (void) SetWindowText(textIDEditBox2, "");
            (void) strcpy_s(currentNPCSelection, "");
            (void) strcpy_s(currentDialogueSelection, "");
            (void) strcpy_s(currentOptionSelection, "");

            // populate the NPC list
            if (SendMessage(dialogueListBox, LB_GETCOUNT, 0, 0) == 0) {
                // add the NPCs to the list
                for (size_t n = 0; n < npcs->size(); n++) {
                    (void) SendMessage(npcListBox, LB_ADDSTRING, 0, (LPARAM) npcs->at(n).GetName());
                };
            };
        };

        // make sure we're not still focused on the button
        (void) SetFocus(npcListBox);
    };

    // Remove NPC button
    ////////////////////

    if (SendMessage(removeNPCButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
        char npcText[FILE_NAME_MAX_SIZE];

        // get the index of the selected NPC text
        LRESULT index = SendMessage(npcListBox, LB_GETCURSEL, 0, 0);
        if (index != LB_ERR && index >= 0) {
            // get the name of the selected NPC
            (void) SendMessage(npcListBox, LB_GETTEXT, (WPARAM) index, (LPARAM) &npcText);

            // remove the NPC from the world
            for (unsigned int n = 0; n < npcs->size(); n++) {
                if (strcmp(npcText, npcs->at(n).GetName()) == 0) {
                    std::swap(npcs->at(n), npcs->back());
                    npcs->pop_back();
                };
            };

            // TODO: remove the created texture

            // update the list box
            (void) SendMessage(npcListBox, LB_DELETESTRING, (WPARAM) index, 0);

            // reset the window
            (void) SendMessage(npcListBox, LB_SETCURSEL, (WPARAM) - 1, 0);
            (void) SetWindowText(nameEditBox, "");
            FreeMemory();
            (void) SendMessage(dialogueListBox, LB_RESETCONTENT, 0, 0);
            (void) SetWindowText(textEditBox, "");
            (void) SetWindowText(textIDEditBox, "");
            (void) SendMessage(optionListBox, LB_RESETCONTENT, 0, 0);
            (void) SetWindowText(textEditBox2, "");
            (void) SetWindowText(textIDEditBox2, "");
            (void) strcpy_s(currentNPCSelection, "");
            (void) strcpy_s(currentDialogueSelection, "");
            (void) strcpy_s(currentOptionSelection, "");
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // user has an NPC selected
    ///////////////////////////

    int currentNPCIndex = (int) SendMessage(npcListBox, LB_GETCURSEL, 0, 0);
    if (currentNPCIndex != LB_ERR) {
        char npcSelection[FILE_NAME_MAX_SIZE];

        // get the name of the current selected NPC
        (void) SendMessage(npcListBox, LB_GETTEXT, (WPARAM) currentNPCIndex, (LPARAM) &npcSelection);

        // user selects a different NPC
        ///////////////////////////////

        if (strcmp(currentNPCSelection, npcSelection) != 0) {
            RECT drawingArea;

            // copy the current selection to previous
            (void) strcpy_s(currentNPCSelection, npcSelection);

            // make sure the editor knows the file location
            (void) strcpy_s(npcSpriteImageLocation, GetNPC(npcs, currentNPCSelection)->GetImageFileLocation());

            // update the window
            char movementSpeed[10];
            (void) _itoa_s(GetNPC(npcs, currentNPCSelection)->GetMovementSpeed(), movementSpeed, sizeof(movementSpeed));
            (void) SetWindowText(nameEditBox, currentNPCSelection);
            (void) SetWindowText(movementSpeedEditBox, movementSpeed);
            (void) SetRect(&drawingArea, 0, 0, 32, 32);
            FreeMemory();
            npcSprite = new VDSprite(direct3DDevice, npcSpriteImageLocation, drawingArea, D3DXVECTOR3((float) npcSpriteDimensions.left, (float) npcSpriteDimensions.top, 0.0f), 0xFF000000);

            // reset the window
            (void) SendMessage(dialogueListBox, LB_RESETCONTENT, 0, 0);
            (void) SetWindowText(textEditBox, "");
            (void) SetWindowText(textIDEditBox, "");
            (void) SendMessage(optionListBox, LB_RESETCONTENT, 0, 0);
            (void) SetWindowText(textEditBox2, "");
            (void) SetWindowText(textIDEditBox2, "");
            (void) strcpy_s(currentDialogueSelection, "");
            (void) strcpy_s(currentOptionSelection, "");

            // populate the dialogue list
            if (SendMessage(dialogueListBox, LB_GETCOUNT, 0, 0) == 0) {
                // get the current NPC
                VDNPC* currentNPC = GetNPC(npcs, currentNPCSelection);

                // add the dialogues to the list
                for (size_t n = 0; n < currentNPC->GetDialoguesSize(); n++) {
                    char dialogue[FILE_NAME_MAX_SIZE];

                    // set it up so it's "ID: Text"
                    (void) _itoa_s(currentNPC->GetDialogue(n)->GetText()->GetID(), dialogue, FILE_NAME_MAX_SIZE, 10);
                    (void) strcat_s(dialogue, ": ");
                    (void) strcat_s(dialogue, currentNPC->GetDialogue(n)->GetText()->GetText());
                    (void) SendMessage(dialogueListBox, LB_ADDSTRING, 0, (LPARAM) dialogue);
                };
            };
        };

        // Save Text (dialogue) button
        //////////////////////////////

        if (SendMessage(saveDialogueButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
            char text[FILE_NAME_MAX_SIZE], textID[FILE_NAME_MAX_SIZE];

            // check for text and ID
            (void) GetWindowText(textEditBox, text, sizeof(text));
            (void) GetWindowText(textIDEditBox, textID, sizeof(textID));
            if (strcmp(text, "") != 0 && strcmp(textID, "") != 0) {
                // add the dialogue
                GetNPC(npcs, currentNPCSelection)->AddDialogue(atoi(textID), text);

                // reset the window
                (void) SendMessage(dialogueListBox, LB_RESETCONTENT, 0, 0);
                (void) SetWindowText(textEditBox, "");
                (void) SetWindowText(textIDEditBox, "");
                (void) SendMessage(optionListBox, LB_RESETCONTENT, 0, 0);
                (void) SetWindowText(textEditBox2, "");
                (void) SetWindowText(textIDEditBox2, "");
                (void) strcpy_s(currentDialogueSelection, "");
                (void) strcpy_s(currentOptionSelection, "");

                // populate the dialogue list
                if (SendMessage(dialogueListBox, LB_GETCOUNT, 0, 0) == 0) {
                    // get the current NPC
                    VDNPC* currentNPC = GetNPC(npcs, currentNPCSelection);

                    // add the dialogues to the list
                    for (size_t n = 0; n < currentNPC->GetDialoguesSize(); n++) {
                        char dialogue[FILE_NAME_MAX_SIZE];

                        // set it up so it's "ID: Text"
                        (void) _itoa_s(currentNPC->GetDialogue(n)->GetText()->GetID(), dialogue, FILE_NAME_MAX_SIZE, 10);
                        (void) strcat_s(dialogue, ": ");
                        (void) strcat_s(dialogue, currentNPC->GetDialogue(n)->GetText()->GetText());
                        (void) SendMessage(dialogueListBox, LB_ADDSTRING, 0, (LPARAM) dialogue);
                    };
                };
            };

            // make sure we're not still focused on the button
            (void) SetFocus(window);
        };

        // Remove Dialogue button
        /////////////////////////

        if (SendMessage(removeDialogueButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
            char text[FILE_NAME_MAX_SIZE];

            // get the index of the selected Dialogue text
            LRESULT index = SendMessage(dialogueListBox, LB_GETCURSEL, 0, 0);
            if (index != LB_ERR && index >= 0) {
                char id[10];

                // get the name of the selected dialogue
                (void) SendMessage(dialogueListBox, LB_GETTEXT, (WPARAM) index, (LPARAM) &text);

                // get the dialogue ID, remove the dialogue from the NPC
                int n = 0;
                while (text[n] != ':') {
                    id[n] = text[n];
                    n++;
                };
                id[n] = '\0';
                GetNPC(npcs, currentNPCSelection)->RemoveDialogue(atoi(id));

                // update the list box
                (void) SendMessage(dialogueListBox, LB_DELETESTRING, (WPARAM) index, 0);
                (void) SendMessage(dialogueListBox, LB_SETCURSEL, (WPARAM) - 1, 0);

                // reset the window
                (void) SetWindowText(textEditBox, "");
                (void) SetWindowText(textIDEditBox, "");
                (void) SendMessage(optionListBox, LB_RESETCONTENT, 0, 0);
                (void) SetWindowText(textEditBox2, "");
                (void) SetWindowText(textIDEditBox2, "");
                (void) strcpy_s(currentDialogueSelection, "");
                (void) strcpy_s(currentOptionSelection, "");
            };

            // make sure we're not still focused on the button
            (void) SetFocus(window);
        };

        // user has a dialogue selected
        ///////////////////////////////

        int currentDialogueIndex = (int) SendMessage(dialogueListBox, LB_GETCURSEL, 0, 0);
        if (currentDialogueIndex != LB_ERR) {
            // get the string of the current selected dialogue
            char dialogueSelection[FILE_NAME_MAX_SIZE];
            (void) SendMessage(dialogueListBox, LB_GETTEXT, (WPARAM) currentDialogueIndex, (LPARAM) &dialogueSelection);

            // get the ID and name of the dialogue
            int n = 0, m = 0;
            char id[10], dialogueName[FILE_NAME_MAX_SIZE];
            while (dialogueSelection[n] != ':') {
                id[n] = dialogueSelection[n];
                n++;
            };
            id[n] = '\0';
            n++;
            while (dialogueSelection[n] != ' ') {
                n++;
            };
            n++;
            while (dialogueSelection[n] != '\0') {
                dialogueName[m] = dialogueSelection[n];
                n++;
                m++;
            };
            dialogueName[m] = '\0';
            int currentDialogueID = atoi(id);

            // user selects a different dialogue
            ////////////////////////////////////

            if (strcmp(currentDialogueSelection, dialogueSelection) != 0) {
                // copy the current selection to previous
                (void) strcpy_s(currentDialogueSelection, dialogueSelection);

                // update the window
                (void) SetWindowText(textEditBox, dialogueName);
                (void) SetWindowText(textIDEditBox, id);
                (void) SendMessage(optionListBox, LB_RESETCONTENT, 0, 0);
                (void) SetWindowText(textEditBox2, "");
                (void) SetWindowText(textIDEditBox2, "");
                (void) strcpy_s(currentOptionSelection, "");

                // populate the options list
                if (SendMessage(optionListBox, LB_GETCOUNT, 0, 0) == 0) {
                    // get the current NPC
                    VDNPC* currentNPC = GetNPC(npcs, currentNPCSelection);

                    // see how many options the dialogue of the NPC has
                    size_t numberOfOptions = currentNPC->FindDialogue(currentDialogueID)->GetOptionsSize();

                    // add the options to the list
                    for (size_t n = 0; n < numberOfOptions; n++) {
                        char option[FILE_NAME_MAX_SIZE];

                        // set it up so it's "ID: Text"
                        (void) _itoa_s(currentNPC->FindDialogue(currentDialogueID)->GetOption(n)->GetNextID(), option, FILE_NAME_MAX_SIZE, 10);
                        (void) strcat_s(option, "[next]: ");
                        (void) strcat_s(option, currentNPC->FindDialogue(currentDialogueID)->GetOption(n)->GetText());
                        (void) SendMessage(optionListBox, LB_ADDSTRING, 0, (LPARAM) option);
                    };
                };
            };

            // Save Text (option) button
            ////////////////////////////

            if (SendMessage(saveOptionButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
                char text[FILE_NAME_MAX_SIZE], nextID[FILE_NAME_MAX_SIZE];

                // check for text and ID
                (void) GetWindowText(textEditBox2, text, sizeof(text));
                (void) GetWindowText(textIDEditBox2, nextID, sizeof(nextID));
                if (strcmp(text, "") != 0 && strcmp(nextID, "") != 0) {
                    // add the option
                    GetNPC(npcs, currentNPCSelection)->AddOption(currentDialogueID, text, atoi(nextID));

                    // reset the window
                    (void) SendMessage(optionListBox, LB_RESETCONTENT, 0, 0);
                    (void) SetWindowText(textEditBox2, "");
                    (void) SetWindowText(textIDEditBox2, "");
                    (void) strcpy_s(currentOptionSelection, "");

                    // populate the options list
                    if (SendMessage(optionListBox, LB_GETCOUNT, 0, 0) == 0) {
                        // get the current NPC
                        VDNPC* currentNPC = GetNPC(npcs, currentNPCSelection);

                        // add the options to the list
                        for (size_t n = 0; n < currentNPC->FindDialogue(currentDialogueID)->GetOptionsSize(); n++) {
                            char option[FILE_NAME_MAX_SIZE];

                            // set it up so it's "ID: Text"
                            (void) _itoa_s(currentNPC->FindDialogue(currentDialogueID)->GetOption(n)->GetNextID(), option, FILE_NAME_MAX_SIZE, 10);
                            (void) strcat_s(option, "[next]: ");
                            (void) strcat_s(option, currentNPC->FindDialogue(currentDialogueID)->GetOption(n)->GetText());
                            (void) SendMessage(optionListBox, LB_ADDSTRING, 0, (LPARAM) option);
                        };
                    };
                };

                // make sure we're not still focused on the button
                (void) SetFocus(window);
            };

            // Remove Response button
            /////////////////////////

            if (SendMessage(removeOptionButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
                char text[FILE_NAME_MAX_SIZE];

                // get the index of the selected Option text
                LRESULT index = SendMessage(optionListBox, LB_GETCURSEL, 0, 0);
                if (index != LB_ERR && index >= 0) {
                    char optionName[FILE_NAME_MAX_SIZE];

                    // get the string of the selected option
                    (void) SendMessage(optionListBox, LB_GETTEXT, (WPARAM) index, (LPARAM) &text);

                    // get the option name, remove the option from the dialogue
                    int n = 0, m = 0;
                    while (text[n] != ' ') {
                        n++;
                    };
                    n++;
                    while (text[n] != '\0') {
                        optionName[m] = text[n];
                        n++;
                        m++;
                    };
                    optionName[m] = '\0';
                    GetNPC(npcs, currentNPCSelection)->RemoveOption(currentDialogueID, optionName);

                    // update the list box
                    (void) SendMessage(optionListBox, LB_DELETESTRING, (WPARAM) index, 0);
                    (void) SendMessage(optionListBox, LB_SETCURSEL, (WPARAM) - 1, 0);

                    // reset the window
                    (void) SetWindowText(textEditBox2, "");
                    (void) SetWindowText(textIDEditBox2, "");
                    (void) strcpy_s(currentOptionSelection, "");
                };

                // make sure we're not still focused on the button
                (void) SetFocus(window);
            };

            // user has an option selected
            //////////////////////////////

            int currentOptionIndex = (int) SendMessage(optionListBox, LB_GETCURSEL, 0, 0);
            if (currentOptionIndex != LB_ERR) {
                // get the name of the current selected option
                char optionSelection[FILE_NAME_MAX_SIZE];
                (void) SendMessage(optionListBox, LB_GETTEXT, (WPARAM) currentOptionIndex, (LPARAM) &optionSelection);

                // user selects a different option
                //////////////////////////////////

                if (strcmp(currentOptionSelection, optionSelection) != 0) {
                    char id[10], optionName[FILE_NAME_MAX_SIZE];

                    // copy the current selection to previous
                    (void) strcpy_s(currentOptionSelection, optionSelection);

                    // get the option ID and name
                    int n = 0, m = 0;
                    while (currentOptionSelection[n] != '[') {
                        id[n] = currentOptionSelection[n];
                        n++;
                    };
                    id[n] = '\0';
                    while (currentOptionSelection[n] != ' ') {
                        n++;
                    };
                    n++;
                    while (currentOptionSelection[n] != '\0') {
                        optionName[m] = currentOptionSelection[n];
                        n++;
                        m++;
                    };
                    optionName[m] = '\0';

                    // update the window
                    (void) SetWindowText(textEditBox2, optionName);
                    (void) SetWindowText(textIDEditBox2, id);
                };
            };
        };
    };
};






/* **********************************************************************************************************************
   AddNPC
   ------
   Adds an NPC to the list.
   #param  npcs               Pointer to the list of NPCs.
   #param  previousName       Only used for changing an existing NPC.
   #param  name               Name of the NPC.
   #param  imageFileLocation  The file location of the sprite.
   #param  movementSpeed      Movement speed of the NPC.
   #return  TRUE if a new NPC was created, FALSE if the image was changed
   **********************************************************************************************************************/

BOOL VDNPCEditor::AddNPC(vector<VDNPC>* npcs, LPCSTR previousName, LPCSTR name, LPCSTR imageFileLocation, int movementSpeed) {
    // see if we're trying to add an NPC that already exists
    for (unsigned int n = 0; n < npcs->size(); n++) {
        if (strcmp(previousName, npcs->at(n).GetName()) == 0) {
            npcs->at(n).SetName(name);
            npcs->at(n).SetImageFileLocation(imageFileLocation);
            npcs->at(n).SetMovementSpeed(movementSpeed);
            return FALSE;
        };
    };

    // add the NPC
    VDNPC newNPC = VDNPC();
    newNPC.SetName(name);
    newNPC.SetImageFileLocation(imageFileLocation);
    newNPC.SetMovementSpeed(movementSpeed);
    npcs->push_back(newNPC);

    return TRUE;
};



/* **********************************************************************************************************************
   WriteNPCs
   ---------
   #param  npcs  Pointer to the list of NPCs.
   Saves the NPC's to hard disk.
   **********************************************************************************************************************/

void VDNPCEditor::WriteNPCs(vector<VDNPC>* npcs) {
    HANDLE file = NULL;
    BOOL returnValue = FALSE;
    DWORD numberOfBytesWritten = 0;
    size_t numberOfNPCs = 0;

    // open the file
    file = CreateFile("Data/NPCs.vd", GENERIC_WRITE, FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    if (file == INVALID_HANDLE_VALUE) {
        FreeMemory();
        VDError("Failed to create the NPCs.vd file.", __FILE__, __LINE__);
    };

    // make sure we let it know how many NPC elements we're adding
    numberOfNPCs = npcs->size();
    returnValue = WriteFile(file, &numberOfNPCs, sizeof(numberOfNPCs), &numberOfBytesWritten, NULL);
    if (returnValue == FALSE) {
        FreeMemory();
        VDError("Failed to write NPC data to file.", __FILE__, __LINE__);
    };

    // write all the NPCs
    for (size_t n = 0; n < numberOfNPCs; n++) {
        npcs->at(n).WriteToFile(file);
        if (returnValue == FALSE) {
            FreeMemory();
            VDError("Failed to write NPC data to file.", __FILE__, __LINE__);
        };
    };

    // close the file
    returnValue = CloseHandle(file);
    if (returnValue == FALSE) {
        FreeMemory();
        VDError("Failed to close the NPCs.vd file.", __FILE__, __LINE__);
    };
};



/* **********************************************************************************************************************
   CreateNPCTexture
   ----------------
   Creates an Nx32 dimension texture of the given sprite and saves it as the given file name.
   #param  direct3DDevice  The Direct3D COM object.
   #param  sprite          The sprite to create a texture from.
   #param  npcFileName     The path and file name of the NPC texture.
   **********************************************************************************************************************/

void VDNPCEditor::CreateNPCTexture(LPDIRECT3DDEVICE9 direct3DDevice, VDSprite* sprite, LPCSTR npcFileName) {
    RECT sourceRect;
    LPDIRECT3DTEXTURE9 tempTexture = NULL;

    // get the extension
    char extension[10];
    int n = 0, m = 0;
    while (npcFileName[n] != '.') {
        n++;
    };
    while (npcFileName[n] != '\0') {
        extension[m] = npcFileName[n];
        n++;
        m++;
    };
    extension[m] = '\0';

    // get the width of the sprite
    UINT sourceSpriteWidth = sprite->GetSpriteWidth();

    // create a texture that's the width of the sprite and 32 pixels in height
    HRESULT returnValue = D3DXCreateTexture(direct3DDevice, sourceSpriteWidth, 32, D3DX_DEFAULT, 0, D3DFMT_X8R8G8B8, D3DPOOL_SCRATCH, &tempTexture);

    // lock the textures
    D3DLOCKED_RECT sourceSprite, destSprite;
    returnValue = sprite->GetTexture()->LockRect(0, &sourceSprite, NULL, D3DLOCK_READONLY);
    if (FAILED(returnValue)) {
        FreeMemory();
        VDDirectXError("Failed to lock the texture.", returnValue, __FILE__, __LINE__);
    }
    returnValue = tempTexture->LockRect(0, &destSprite, NULL, D3DLOCK_DISCARD);
    if (FAILED(returnValue)) {
        sprite->GetTexture()->UnlockRect(0);
        FreeMemory();
        VDDirectXError("Failed to lock the texture.", returnValue, __FILE__, __LINE__);
    };

    // copy the data
    UINT bytesPerPixel = 4;
    sprite->GetSourceRectangle(&sourceRect);
    const BYTE* sourceLoc = (const BYTE*) sourceSprite.pBits + sourceRect.top * sourceSpriteWidth * bytesPerPixel;
    for (int n = 0; n < 32; n++) {
        BYTE* destAddress = (BYTE *) destSprite.pBits + n * destSprite.Pitch;
        memcpy(destAddress, sourceLoc, sourceSpriteWidth * bytesPerPixel);
        sourceLoc += sourceSprite.Pitch;
    };

    // unlock the textures
    sprite->GetTexture()->UnlockRect(0);
    tempTexture->UnlockRect(0);

    // save the texture to hard disk - requires a directory Images to be in the app directory
    if (strcmp(extension, ".png") == 0) {
        returnValue = D3DXSaveTextureToFile(npcFileName, D3DXIFF_PNG, tempTexture, NULL);
        if (FAILED(returnValue)) {
            FreeMemory();
            VDDirectXError("Failed to save texture.", returnValue, __FILE__, __LINE__);
        };
    } else if (strcmp(extension, ".bmp") == 0) {
        returnValue = D3DXSaveTextureToFile(npcFileName, D3DXIFF_BMP, tempTexture, NULL);
        if (FAILED(returnValue)) {
            FreeMemory();
            VDDirectXError("Failed to save texture.", returnValue, __FILE__, __LINE__);
        };
    };

    // free memory
    if (tempTexture != NULL) {
        tempTexture->Release();
        tempTexture = NULL;
    };
};



/* **********************************************************************************************************************
   Accessors and Modifiers
   **********************************************************************************************************************/

VDNPC* VDNPCEditor::GetNPC(vector<VDNPC>* npcs, LPCSTR name) {
    for (unsigned int n = 0; n < npcs->size(); n++) {
        if (strcmp(name, npcs->at(n).GetName()) == 0) {
            return &npcs->at(n);
        };
    };

    return NULL;
};