/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "VDObjectEditor.h"



/* **********************************************************************************************************************
   Constructor
   **********************************************************************************************************************/

VDObjectEditor::VDObjectEditor() {
    window = NULL;
    objectListBox = NULL;
    (void) strcpy_s(currentObjectSelection, "");
    objectSprite = NULL;
    (void) strcpy_s(objectSpriteImageLocation, "");
    (void) SecureZeroMemory(&objectSpriteDimensions, sizeof(objectSpriteDimensions));
    spriteSelected = FALSE;
    removeObjectButton = NULL;
    loadSpriteButton = NULL;
    spriteSheet = NULL;
    (void) strcpy_s(spriteSheetImageLocation, "");
    (void) SecureZeroMemory(&spriteSheetDimensions, sizeof(spriteSheetDimensions));
    (void) SecureZeroMemory(&spriteSheetSourceRectangle, sizeof(spriteSheetSourceRectangle));
    (void) strcpy_s(selectedSpriteImageLocation, "");
    shiftSpriteSheetUp = NULL;
    shiftSpriteSheetDown = NULL;
    shiftSpriteSheetLeft = NULL;
    shiftSpriteSheetRight = NULL;
    nameEditBox = NULL;
    addObjectButton = NULL;
    reloadObjects = FALSE;
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

VDObjectEditor::~VDObjectEditor() {
};



void VDObjectEditor::FreeMemory() {
    if (spriteSheet != NULL) {
        spriteSheet->FreeTexture();
        delete spriteSheet;
        spriteSheet = NULL;
    };
    if (objectSprite != NULL) {
        objectSprite->FreeTexture();
        delete objectSprite;
        objectSprite = NULL;
    };
};



/* **********************************************************************************************************************
   Load
   TODO
   **********************************************************************************************************************/

HWND VDObjectEditor::Load(HINSTANCE instance, HWND parentWindow, LPCSTR className) {
    RECT windowDimensions;
    (void) GetWindowRect(parentWindow, &windowDimensions);
    window = CreateWindowEx(WS_EX_TOPMOST | WS_EX_TOOLWINDOW,      // extended window styles
                            className,                             // class name
                            "Object Editor",                       // window name
                            WS_CAPTION | WS_SYSMENU | WS_VISIBLE,  // window styles
                            windowDimensions.left + 100,           // x coordinate (0 = left)
                            windowDimensions.top + 100,            // y coordinate (0 = top)
                            255,                                   // width
                            500,                                   // height
                            NULL,                                  // parent/owner window
                            NULL,                                  // menu
                            instance,                              // instance
                            NULL);                                 // used for CREATESTRUCT structure
    if (window == NULL) {
        VDError("Failed to create the window.", __FILE__, __LINE__);
    };

    // objects
    HWND objectListStatic = CreateWindow("Static", "Objects:", WS_CHILD | WS_VISIBLE, 10, 10, 60, 18, window, (HMENU) VD_OBJECT_EDITOR_STATIC_OBJECT_LIST, instance, NULL);
    if (objectListStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    objectListBox = CreateWindow("ListBox", NULL, WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL, 10, 30, 100, 100, window, (HMENU) VD_OBJECT_EDITOR_OBJECT_LIST, instance, NULL);
    if (objectListBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    (void) SetRect(&objectSpriteDimensions, 120, 30, 152, 62);
    removeObjectButton = CreateWindow("Button", "Remove Object", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 10, 133, 110, 20, window, (HMENU) VD_OBJECT_EDITOR_REMOVE_OBJECT, instance, NULL);
    if (removeObjectButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    // sprite sheet part
    loadSpriteButton = CreateWindow("Button", "Load Sprite Sheet", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 10, 185, 125, 20, window, (HMENU) VD_OBJECT_EDITOR_LOAD_SPRITE, instance, NULL);
    if (loadSpriteButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    (void) strcpy_s(spriteSheetImageLocation, "");
    (void) SetRect(&spriteSheetDimensions, 10, 215, 170, 375);
    (void) SetRect(&spriteSheetSourceRectangle, 0, 0, 32, 32);
    (void) strcpy_s(selectedSpriteImageLocation, "");
    shiftSpriteSheetUp = CreateWindow("Button", "/\\", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 170, 215, 20, 20, window, (HMENU) VD_OBJECT_EDITOR_UP, instance, NULL);
    if (shiftSpriteSheetUp == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    shiftSpriteSheetDown = CreateWindow("Button", "\\/", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 170, 355, 20, 20, window, (HMENU) VD_OBJECT_EDITOR_DOWN, instance, NULL);
    if (shiftSpriteSheetDown == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    shiftSpriteSheetLeft = CreateWindow("Button", "<", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 10, 375, 20, 20, window, (HMENU) VD_OBJECT_EDITOR_LEFT, instance, NULL);
    if (shiftSpriteSheetLeft == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    shiftSpriteSheetRight = CreateWindow("Button", ">", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 150, 375, 20, 20, window, (HMENU) VD_OBJECT_EDITOR_RIGHT, instance, NULL);
    if (shiftSpriteSheetRight == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    // adding object
    HWND objectNameStatic = CreateWindow("Static", "Name:", WS_CHILD | WS_VISIBLE, 10, 400, 50, 18, window, (HMENU) VD_OBJECT_EDITOR_STATIC_NAME, instance, NULL);
    if (objectNameStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    nameEditBox = CreateWindow("Edit", "", WS_CHILD | WS_BORDER | WS_VISIBLE, 60, 400, 140, 18, window, (HMENU) VD_OBJECT_EDITOR_NAME, instance, NULL);
    if (nameEditBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    addObjectButton = CreateWindow("Button", "Add", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 205, 400, 35, 20, window, (HMENU) VD_OBJECT_EDITOR_ADD, instance, NULL);
    if (addObjectButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    saveChangesButton = CreateWindow("Button", "SAVE CHANGES", WS_CHILD | BS_PUSHBUTTON | WS_BORDER | WS_VISIBLE, 55, 427, 130, 40, window, (HMENU) VD_OBJECT_EDITOR_SAVE_CHANGES, instance, NULL);
    if (saveChangesButton == NULL) {
        FreeMemory();
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    return window;
};



// TODO

void VDObjectEditor::ProcessMessages(LPDIRECT3DDEVICE9 direct3DDevice, VDInput* input, vector<VDEntity>* objects) {
    // set the input to the Object Editor
    if (input->GetCurrentWindow() != window) {
        input->ChangeInputWindow(window);
    };

    // populate the object list if it's empty
    if (SendMessage(objectListBox, LB_GETCOUNT, 0, 0) == 0) {
        for (size_t n = 0; n < objects->size(); n++) {
            (void) SendMessage(objectListBox, LB_ADDSTRING, 0, (LPARAM) objects->at(n).GetName());
        };
    };

    // user has an object selected
    //////////////////////////////

    int currentObjectIndex = (int) SendMessage(objectListBox, LB_GETCURSEL, 0, 0);
    if (currentObjectIndex != LB_ERR) {
        // get the name of the current selected object
        char objectSelection[FILE_NAME_MAX_SIZE];
        (void) SendMessage(objectListBox, LB_GETTEXT, (WPARAM) currentObjectIndex, (LPARAM) &objectSelection);

        // user selects a different object
        //////////////////////////////////

        if (strcmp(currentObjectSelection, objectSelection) != 0) {
            // copy the current selection
            (void) strcpy_s(currentObjectSelection, objectSelection);

            // get the path and file name for the selected object
            (void) strcpy_s(objectSpriteImageLocation, GetVDObject(objects, currentObjectSelection)->GetImageFileLocation());

            // create a sprite for the object
            RECT drawingArea;
            (void) SetRect(&drawingArea, 0, 0, 32, 32);
            if (objectSprite != NULL) {
                objectSprite->FreeTexture();
                delete objectSprite;
                objectSprite = NULL;
            };
            objectSprite = new VDSprite(direct3DDevice, objectSpriteImageLocation, drawingArea, D3DXVECTOR3((float) objectSpriteDimensions.left, (float) objectSpriteDimensions.top, 0.0f), 0xFF000000);
        };
    };

    // Remove Object button
    ///////////////////////

    if (SendMessage(removeObjectButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
        char objectText[FILE_NAME_MAX_SIZE];

        // get the index of the selected object text
        LRESULT index = SendMessage(objectListBox, LB_GETCURSEL, 0, 0);
        if (index != LB_ERR && index >= 0) {
            // get the name of the selected object
            (void) SendMessage(objectListBox, LB_GETTEXT, (WPARAM) index, (LPARAM) &objectText);

            // remove the object from the world
            for (unsigned int n = 0; n < objects->size(); n++) {
                if (strcmp(objectText, objects->at(n).GetName()) == 0) {
                    std::swap(objects->at(n), objects->back());
                    objects->pop_back();
                };
            };

            // TODO: remove the created texture

            // update the list box
            (void) SendMessage(objectListBox, LB_DELETESTRING, (WPARAM) index, 0);

            // reset the window
            FreeMemory();
            (void) SendMessage(objectListBox, LB_SETCURSEL, (WPARAM) - 1, 0);
            (void) SetWindowText(nameEditBox, "");
            (void) strcpy_s(currentObjectSelection, "");
            spriteSelected = FALSE;
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Load Sprite Sheet button
    ///////////////////////////

    if (SendMessage(loadSpriteButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
        char text[FILE_NAME_MAX_SIZE];

        // open a sprite image
        VDOpenFile(window, "Sprites (*.png,*.bmp)\0*.png;*.bmp\0All Files\0*.*\0", text);

        // make sure the user selected a file
        if (strcmp(text, "") != 0) {
            // set the sprite file location
            (void) strcpy_s(spriteSheetImageLocation, text);

            // clear the selected sprite stuff
            (void) SendMessage(objectListBox, LB_SETCURSEL, (WPARAM) - 1, 0);
            if (objectSprite != NULL) {
                objectSprite->FreeTexture();
                delete objectSprite;
                objectSprite = NULL;
            };
            (void) strcpy_s(objectSpriteImageLocation, "");
            spriteSelected = FALSE;

            // add the sprite to the editor so it may be drawn
            RECT drawingArea;
            (void) SetRect(&drawingArea, 0, 0, 160, 160);
            FreeMemory();
            spriteSheet = new VDSprite(direct3DDevice, text, drawingArea, D3DXVECTOR3((float) spriteSheetDimensions.left, (float) spriteSheetDimensions.top, 0.0f), 0xFF000000);
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Up button
    ////////////

    if (SendMessage(shiftSpriteSheetUp, BM_GETSTATE, 0, 0) == BST_FOCUS || input->GetKeyboardInput()[DIK_UP] & 0x80) {
        if (spriteSheet != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) spriteSheet->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            if (drawingArea.top == 0) {
                LONG bottom = spriteSheet->GetSpriteHeight();
                spriteSheet->SetSourceRectangle(drawingArea.left, bottom - 160, drawingArea.right, bottom);
            } else {
                spriteSheet->SetSourceRectangle(drawingArea.left, drawingArea.top - 32, drawingArea.right, drawingArea.bottom - 32);
            };

            // let the map editor know we need to reselect
            spriteSelected = FALSE;
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Down button
    //////////////

    if (SendMessage(shiftSpriteSheetDown, BM_GETSTATE, 0, 0) == BST_FOCUS || input->GetKeyboardInput()[DIK_DOWN] & 0x80) {
        if (spriteSheet != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) spriteSheet->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            LONG bottom = spriteSheet->GetSpriteHeight();
            if (drawingArea.bottom == bottom) {
                spriteSheet->SetSourceRectangle(drawingArea.left, 0, drawingArea.right, 160);
            } else {
                spriteSheet->SetSourceRectangle(drawingArea.left, drawingArea.top + 32, drawingArea.right, drawingArea.bottom + 32);
            };

            // let the map editor know we need to reselect
            spriteSelected = FALSE;
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Left button
    //////////////

    if (SendMessage(shiftSpriteSheetLeft, BM_GETSTATE, 0, 0) == BST_FOCUS || input->GetKeyboardInput()[DIK_LEFT] & 0x80) {
        if (spriteSheet != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) spriteSheet->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            if (drawingArea.left == 0) {
                LONG right = spriteSheet->GetSpriteWidth();
                spriteSheet->SetSourceRectangle(right - 160, drawingArea.top, right, drawingArea.bottom);
            } else {
                spriteSheet->SetSourceRectangle(drawingArea.left - 32, drawingArea.top, drawingArea.right - 32, drawingArea.bottom);
            };

            // let the map editor know we need to reselect
            spriteSelected = FALSE;
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Right button
    ///////////////

    if (SendMessage(shiftSpriteSheetRight, BM_GETSTATE, 0, 0) == BST_FOCUS || input->GetKeyboardInput()[DIK_RIGHT] & 0x80) {
        if (spriteSheet != NULL) {
            // get the sprite's source rectangle
            RECT drawingArea;
            (void) spriteSheet->GetSourceRectangle(&drawingArea);

            // see if we're wrapping around
            LONG right = spriteSheet->GetSpriteWidth();
            if (drawingArea.right == right) {
                spriteSheet->SetSourceRectangle(0, drawingArea.top, 160, drawingArea.bottom);
            } else {
                spriteSheet->SetSourceRectangle(drawingArea.left + 32, drawingArea.top, drawingArea.right + 32, drawingArea.bottom);
            };

            // let the map editor know we need to reselect
            spriteSelected = FALSE;
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // user selects a tile
    //////////////////////

    if (spriteSheet != NULL) {
        // get the current cursor position
        POINT mousePosition;
        GetCursorPos(&mousePosition);

        // take a look at the Map Editor window location
        RECT windowDimensions;
        (void) GetWindowRect(window, &windowDimensions);

        // acquire the real mouse position
        POINT objectEditorMousePosition;
        objectEditorMousePosition.x = mousePosition.x - GetSystemMetrics(SM_CXFIXEDFRAME) - windowDimensions.left;
        objectEditorMousePosition.y = mousePosition.y - (GetSystemMetrics(SM_CYFIXEDFRAME) + GetSystemMetrics(SM_CYSMCAPTION)) - windowDimensions.top;

        // see if cursor is in the sprite area
        if (objectEditorMousePosition.x >= spriteSheetDimensions.left && objectEditorMousePosition.x < spriteSheetDimensions.right && objectEditorMousePosition.y >= spriteSheetDimensions.top && objectEditorMousePosition.y < spriteSheetDimensions.bottom) {
            // user clicks on the sprite sheet
            if (input->GetMouseInput().rgbButtons[0] & 0x80) {
                // let the map editor know we selected a tile
                spriteSelected = TRUE;

                // set up the tile source rectangle
                objectEditorMousePosition.x -= spriteSheetDimensions.left;
                objectEditorMousePosition.y -= spriteSheetDimensions.top;
                LONG tileSelectedX = objectEditorMousePosition.x - (objectEditorMousePosition.x % 32);
                LONG tileSelectedY = objectEditorMousePosition.y - (objectEditorMousePosition.y % 32);

                // get the current source rectangle selection
                RECT currentSelection;
                spriteSheet->GetSourceRectangle(&currentSelection);
                currentSelection.left += tileSelectedX;
                currentSelection.top += tileSelectedY;
                currentSelection.right = currentSelection.left + 32;
                currentSelection.bottom = currentSelection.top + 32;

                // make sure we selected a different 32x32 sprite area
                if (spriteSheetSourceRectangle.left != currentSelection.left || spriteSheetSourceRectangle.top != currentSelection.top || spriteSheetSourceRectangle.right != currentSelection.right || spriteSheetSourceRectangle.bottom != currentSelection.bottom) {
                    // set the new 32x32 source rectangle
                    spriteSheetSourceRectangle.left = currentSelection.left;
                    spriteSheetSourceRectangle.top = currentSelection.top;
                    spriteSheetSourceRectangle.right = currentSelection.right;
                    spriteSheetSourceRectangle.bottom = currentSelection.bottom; 

                    // set selected tile to an empty string since we might have to create a new texture
                    // TODO: create the texture when we click Add, copy the new name to the following:
                    //(void) strcpy_s(currentObjectSelection, "");
                };
            };
        };
    };

    // Add button
    /////////////

    if (SendMessage(addObjectButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
        // make sure we have a sprite selected
        if (spriteSelected == TRUE) {
            // check for a name
            char objectName[FILE_NAME_MAX_SIZE];
            (void) GetWindowText(nameEditBox, objectName, sizeof(objectName));
            if (strcmp(objectName, "") != 0) {
                char newImageFile[FILE_NAME_MAX_SIZE], extension[10];

                // remove the path name, save the extension
                int n = 0, m = 0;
                while (spriteSheetImageLocation[n] != '.') {
                    n++;
                };
                while (spriteSheetImageLocation[n] != '\0') {
                    extension[m] = spriteSheetImageLocation[n];
                    n++;
                    m++;
                };
                extension[m] = '\0';

                // use the object name for the new texture
                (void) strcpy_s(newImageFile, "Images/");
                (void) strcat_s(newImageFile, objectName);
                (void) strcat_s(newImageFile, extension);

                // create the texture if it doesn't exist
                CreateTexture(direct3DDevice, newImageFile);

                // add the object
                if (AddObject(objects, objectName, newImageFile) == TRUE) {
                    (void) SendMessage(objectListBox, LB_ADDSTRING, 0, (LPARAM) objectName);
                };

                // reset the window
                (void) SendMessage(objectListBox, LB_SETCURSEL, (WPARAM) - 1, 0);
                (void) SetWindowText(nameEditBox, "");
                (void) strcpy_s(currentObjectSelection, "");
            };
        };

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };

    // Save Changes button
    //////////////////////

    if (SendMessage(saveChangesButton, BM_GETSTATE, 0, 0) == BST_FOCUS) {
        // save the objects
        WriteObjects(objects);

        // lets other editors know we've changed objects
        reloadObjects = TRUE;

        // make sure we're not still focused on the button
        (void) SetFocus(window);
    };
};



BOOL VDObjectEditor::AddObject(vector<VDEntity>* objects, LPCSTR name, LPCSTR imageFileLocation) {
    // see if we're trying to add an object that already exists
    for (unsigned int n = 0; n < objects->size(); n++) {
        if (strcmp(name, objects->at(n).GetName()) == 0) {
            objects->at(n).SetImageFileLocation(imageFileLocation);
            return FALSE;
        };
    };

    // add the object
    VDEntity newObject;
    newObject.SetName(name);
    newObject.SetImageFileLocation(imageFileLocation);
    objects->push_back(newObject);

    return TRUE;
};



void VDObjectEditor::CreateTexture(LPDIRECT3DDEVICE9 direct3DDevice, LPCSTR objectFileName) {
    LPDIRECT3DTEXTURE9 tempTexture = NULL;

    // get the extension
    char extension[10];
    int n = 0, m = 0;
    while (objectFileName[n] != '.') {
        n++;
    };
    while (objectFileName[n] != '\0') {
        extension[m] = objectFileName[n];
        n++;
        m++;
    };
    extension[m] = '\0';

    // create a new 32x32 sprite
    HRESULT returnValue = D3DXCreateTexture(direct3DDevice, 32, 32, D3DX_DEFAULT, 0, D3DFMT_X8R8G8B8, D3DPOOL_SCRATCH, &tempTexture);

    // lock the textures
    D3DLOCKED_RECT sourceSprite, destSprite;
    returnValue = spriteSheet->GetTexture()->LockRect(0, &sourceSprite, NULL, D3DLOCK_READONLY);
    if (FAILED(returnValue)) {
        FreeMemory();
        VDDirectXError("Failed to lock the texture.", returnValue, __FILE__, __LINE__);
    }
    returnValue = tempTexture->LockRect(0, &destSprite, NULL, D3DLOCK_DISCARD);
    if (FAILED(returnValue)) {
        spriteSheet->GetTexture()->UnlockRect(0);
        FreeMemory();
        VDDirectXError("Failed to lock the texture.", returnValue, __FILE__, __LINE__);
    };

    // copy the data from the start of the source rectangle in the texture to the end
    UINT sourceSpriteWidth = spriteSheet->GetSpriteWidth();
    UINT bytesPerPixel = 4;
    const BYTE* sourceLoc = (const BYTE*) sourceSprite.pBits + spriteSheetSourceRectangle.top * sourceSpriteWidth * bytesPerPixel
                                                             + spriteSheetSourceRectangle.left * bytesPerPixel;
    for (int n = 0; n < 32; n++) {
        BYTE* destAddress = (BYTE *) destSprite.pBits + n * destSprite.Pitch;
        memcpy(destAddress, sourceLoc, 32 * bytesPerPixel);
        sourceLoc += sourceSprite.Pitch;
    };

    // unlock the textures
    spriteSheet->GetTexture()->UnlockRect(0);
    tempTexture->UnlockRect(0);

    // save the texture to hard disk - requires a directory Images to be in the app directory
    if (strcmp(extension, ".png") == 0) {
        returnValue = D3DXSaveTextureToFile(objectFileName, D3DXIFF_PNG, tempTexture, NULL);
        if (FAILED(returnValue)) {
            FreeMemory();
            VDDirectXError("Failed to save texture.", returnValue, __FILE__, __LINE__);
        };
    } else if (strcmp(extension, ".bmp") == 0) {
        returnValue = D3DXSaveTextureToFile(objectFileName, D3DXIFF_BMP, tempTexture, NULL);
        if (FAILED(returnValue)) {
            FreeMemory();
            VDDirectXError("Failed to save texture.", returnValue, __FILE__, __LINE__);
        };
    };

    // free memory
    if (tempTexture != NULL) {
        tempTexture->Release();
        tempTexture = NULL;
    };
};



void VDObjectEditor::WriteObjects(vector<VDEntity>* objects) {
    HANDLE file = NULL;
    BOOL returnValue = FALSE;
    DWORD numberOfBytesWritten = 0;
    size_t numberOfObjects = 0;

    // open the file
    file = CreateFile("Data/Objects.vd", GENERIC_WRITE, FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    if (file == INVALID_HANDLE_VALUE) {
        FreeMemory();
        VDError("Failed to create the Objects.vd file.", __FILE__, __LINE__);
    };

    // make sure we let it know how many objects we're adding
    numberOfObjects = objects->size();
    returnValue = WriteFile(file, &numberOfObjects, sizeof(numberOfObjects), &numberOfBytesWritten, NULL);
    if (returnValue == FALSE) {
        FreeMemory();
        VDError("Failed to write object data to file.", __FILE__, __LINE__);
    };

    // write all the NPCs
    for (size_t n = 0; n < numberOfObjects; n++) {
        objects->at(n).WriteToFile(file);
        if (returnValue == FALSE) {
            FreeMemory();
            VDError("Failed to write object data to file.", __FILE__, __LINE__);
        };
    };

    // close the file
    returnValue = CloseHandle(file);
    if (returnValue == FALSE) {
        FreeMemory();
        VDError("Failed to close the Objects.vd file.", __FILE__, __LINE__);
    };
};



VDEntity* VDObjectEditor::GetVDObject(vector<VDEntity>* objects, LPCSTR name) {
    for (unsigned int n = 0; n < objects->size(); n++) {
        if (strcmp(name, objects->at(n).GetName()) == 0) {
            return &objects->at(n);
        };
    };

    return NULL;
};